
import com.mhsoftware.cdaily.business.calendar;
import com.mhsoftware.cdaily.business.calitem;
import com.mhsoftware.cdaily.business.calitemtypes;
import com.mhsoftware.cdaily.business.User;
import com.mhsoftware.cdaily.support.WebApp;
import com.MHSoftware.dates.InvalidDateException;
import com.MHSoftware.dates.SaneDate;
import java.io.FileInputStream;
import java.io.IOException;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.Properties;

/**	Sample code to show adding calendar items programmatically.
 *
 * This demonstrates adding one time, weekly, and monthly recurring item types.
 * Usage: java -classpath cdaily.jar:MHS.jar itemAddDemo PathToConfiguration.configuration.properties
 */
public class itemAddDemo {

	public static void main(String argv[]) throws SQLException,IOException,InvalidDateException {

	//	The middle tier classes read these properties from the WebApp.class.
	//	Since WebApp is normally initialized by the servlet engine, we have
	//	to do this ourselves.
	WebApp.setPropertiesFile(argv[0]);
	User u=WebApp.getPublicUser(null);
	System.out.println("All WebApp settings completed.");

	//	The calendar ID we will use is the first calendar in the system. In your code, you will
	//	need to pick the correct calendar ID to use for the calitem.
	calendar cal=(calendar)calendar.getItems().elementAt(0);

	//	Just pick up an item type ID
	calitemtypes ciType=(calitemtypes)calitemtypes.getItemsUserCanUse(u.user_id).elementAt(0);
	System.out.println("Adding new items to calendar "+cal.name+" ID="+cal.calendar_id);


	/*	Demonstrate adding a one-time event.
	*/
	calitem ci=new calitem(cal);
	ci.owner = u.user_id;
	ci.item_type_id=ciType.item_type_id;
	ci.description="This is a test added item.";
	//
	//	Make this a one time item.
	//
	ci.setType(calitem.TYPE_ONETIME);
	//	Set the starting date to April 15 2003
    ci.starting_date=new SaneDate("2003-04-15",SaneDate.FORMAT_YYYY_MM_DD);
	//	Set the ending date to be start day + 6 Days.
	ci.ending_date=new SaneDate(ci.starting_date.getJulianDate()+6);
	ci.update();
	System.out.println("The new calitem ID for "+ci.description+" is "+ci.cal_item_id);


	/*
		Demonstrate adding a recurring event weekly with Exception dates.
	*/
	ci=new calitem(cal);
	ci.owner = u.user_id;
	ci.description="Choir Practice";
	ci.item_type_id=ciType.item_type_id;
	ci.setType(calitem.TYPE_WEEKLY);
	ci.setDOW(calitem.DAY_WEDNESDAY);
	ci.starting_time="7:00 PM";
	ci.ending_time="8:15 PM";
	// No Choir practice on Christmas day.
	ci.addException(new SaneDate("2002-12-25",SaneDate.FORMAT_YYYY_MM_DD));
	//	or New years day.
	ci.addException(new SaneDate("2003-01-01",SaneDate.FORMAT_YYYY_MM_DD));
	ci.long_description="Come join our celebration choir.";
	ci.update();
	System.out.println("The new calendar item ID for "+ci.description+" is "+ci.cal_item_id);


	/*
		Finally, demonstrate adding a 4th thursday event starting 1 November 2002, and ending 1 june 2003
	*/
	ci=new calitem(cal);
	ci.item_type_id=ciType.item_type_id;
	ci.owner = u.user_id;
	ci.description="4th Thursday add test";
	ci.setType(calitem.TYPE_MONTHLY);
	ci.setRecursOrdinal(calitem.ORDINAL_FOURTH);
	ci.setDOW(calitem.DAY_THURSDAY);
	ci.starting_date=new SaneDate("2002-11-01",SaneDate.FORMAT_YYYY_MM_DD);
	ci.ending_date=new SaneDate("2003-06-01",SaneDate.FORMAT_YYYY_MM_DD);
	ci.update();
	System.out.println("The new calendar item ID for "+ci.description+" is "+ci.cal_item_id);
	System.out.println("Concluding Item Add demonstration");
	}
}
