package com.mhsoftware.cdaily.support.security;
/*
	Connect Daily Web Calendar Software

	Copyright 2008, MH Software, Inc. All Rights Reserved
	5023 W 120th Ave #311, Broomfield CO 80020
	+1 303 438 9585

	Last Edit By: "$Author: gsexton $"
	Last Checkin: "$Date: 2012/12/01 18:12:12 $"
	Revision #  : "$Revision: 1.5 $"

*/

import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;
import javax.servlet.ServletContext;

import com.mhsoftware.cdaily.business.User;
import com.mhsoftware.cdaily.support.WebApp;
/** 
* This class provides a dummy authentication provider that works 
* with container level security. 
*  
* In essence, it verifies that login pre-conditions are OK, and 
* if so returns true. It does no actual verification of the 
* password because that's being handled by the container. 
*
*   @author George Sexton
*  
*   @since 3.4.0
*/

public class AuthProviderContainer extends AuthenticationProvider {

	public static void init(ServletContext sc){
		context=sc;
		provider_flags=AuthenticationProvider.CONTAINER_SECURITY | 
			AuthenticationProvider.DOES_AUTO_ADD;
		if (isDebugMode()) {
			System.out.println("Using Container Authentication Provider!");
		}
	}

	public LoginResult authenticateUser(String UserID, String Password,String TransFormSeed,ResourceBundle rb,Locale lc) {
		throw new IllegalStateException("This method is not supported by AuthProviderContainer!");
	}

	public LoginResult authenticateUser(String UserID, String Password,ResourceBundle rb, Locale lc) {
		if (isDebugMode()) {
			System.out.println("AuthProviderContainer.authenticateUser(\""+UserID+"\",\""+Password+"\",rb,lc)");
		}
		if (UserID.indexOf('\\')>0) {
			String[] aParts=UserID.split("\\\\");
			UserID=aParts[aParts.length-1];
			if (isDebugMode()) {
				System.out.println("Shortened User ID to :"+UserID);
			}
		}
		lr=new LoginResult();
		User u=User.getUserByName(UserID);
		if (u==null) {
			/*
				If the system is configured to auto-add the user, then auto-
				create the account.
            */
			String sTemplate=WebApp.getProperty("Security.AutoAddUserTemplate");
			if (sTemplate!=null) {
				u=User.createUserFromTemplate(UserID,sTemplate);
				lr.setUser(u);
			}
		}
		if (u==null) {
			lr.getFatalList().add(rb.getString("InvalidUserOrPW"));
		} else {
			lr.setUser(u);
			List l=verifyLoginPreconditions(u,rb);
			if (l.isEmpty()) {
				processSuccessfulLogin(u,rb,lc);
			}

		}
		return lr;
	}
}


