package com.mhsoftware.cdaily.support.security;
/*
	Connect Daily Web Calendar Software

	Copyright 2002-2005, MH Software, Inc. All Rights Reserved
	5023 W 120th Ave #311, Broomfield CO 80020
	+1 303 438 9585

	Last Edit By: "$Author: gsexton $"
	Last Checkin: "$Date: 2009/04/18 19:19:22 $"
	Revision #  : "$Revision: 1.10 $"

*/
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.servlet.ServletContext;

import com.mhsoftware.cdaily.business.User;

/**
*	This class implements AuthenticationProvider
*	and is the same as the DefaultAuthProvider,
*	except that it hashes the user password using
*	the MD5 cryptographic hash prior to doing the
*	database lookup.
 *
*	@author George Sexton
*/

public class AuthProviderMD5 extends AuthProviderDefault {

	public static void init(ServletContext sc){
		context=sc;
		provider_flags=AuthenticationProvider.CHANGES_PASSWORDS|AuthenticationProvider.DO_PRETRANSFORM;
	}

	/**
	 * This is an authentication method where the password supplied by the user
	 * has been transformed using an MD5Sum and another seed value.
	 * @param UserID The User Name to authenticate
	 * @param Password the Transformed Password.
	 * @param TransformSeed Seed value used in the transformation of the
	 * password.
	 * @param rb Resource Bundle for formatting messages.
	 * @param lc Locale used for formatting messages.
	 */
	public LoginResult authenticateUser(String UserID, String Password,String TransformSeed,ResourceBundle rb,Locale lc) {
		/*
			The password we receive is actually md5(md5(seed)+md5(password)), so to check the password we need to

			md5(md5(seed)+userObject.password) and compare it to the password parameter.
		*/
		lr=new LoginResult();
		User u=User.getUserByName(UserID);
		if (u==null) {
			lr.getFatalList().add(rb.getString("InvalidUserOrPW"));
		} else {
			lr.setUser(u);
			List l=verifyLoginPreconditions(u,rb);
			if (l.isEmpty()) {
				String sPassword=transformPassword(transformPassword(TransformSeed)+u.password);
				if (sPassword.equals(Password)) {
					processSuccessfulLogin(u,rb,lc);
				} else {
					processFailedLogin(u,rb);
				}
			}

		}
		return lr;
	}

	public String transformPassword(final String Password){
		if (Password==null) {
			return null;
		} else {
			try {
				return com.MHSoftware.string.util.toHexString(MessageDigest.getInstance("MD5").digest(Password.trim().getBytes())).toLowerCase();
			} catch (NoSuchAlgorithmException e) {
				if (context==null) {
					e.printStackTrace();
				} else {
					context.log("AuthProviderMD5",e);
				}
				return null;
			}
		}
	}
}
