/*
	Connect Daily Web Calendar Software
	
	Copyright 2002, MH Software, Inc. All Rights Reserved
	5023 W 120th Ave #311, Broomfield CO 80020
	+1 303 438 9585
*/

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;
import java.util.Vector;


import com.MHSoftware.net.ConnectionPool.MHdbConn;
import com.MHSoftware.net.ConnectionPool.MHdbPool;
import com.MHSoftware.net.ConnectionPool.MHdbQueue;
import com.mhsoftware.cdaily.business.calitemtypes;
import com.mhsoftware.cdaily.business.candidatetokens;
import com.mhsoftware.cdaily.business.groups;
import com.mhsoftware.cdaily.business.groupusers;
import com.mhsoftware.cdaily.business.objectsecurity;
import com.mhsoftware.cdaily.business.resource;
import com.mhsoftware.cdaily.business.resourcetype;
import com.mhsoftware.cdaily.support.*;

/**
 * Schema conversion class for updating to schema version 2.5.0.
 *
 * @author George Sexton
 * @author Christopher Cain
 */
public class AccessSchema250 {

	public static final String PROC_NAME = "ConnectDaily Schema Update";
	public static final String ERR_PREFIX = "Non-Fatal Error ...";
	public static final String ERR_PREFIX_FATAL = "Fatal Error!";
	public static final String INFO_PREFIX = "Info: ";
	public static final String ABORT_MSG = "Update aborted!";
	public static final String CMD_TERM_CHAR = "~";

	public static final int TYPE_ID_CALENDAR =       32768;
	public static final int TYPE_ID_CALGROUP =       32769;
	public static final int TYPE_ID_RESOURCETYPE =   32770;
	public static final int TYPE_ID_RESOURCES =      32771;
	public static final int TYPE_ID_CALITEMTYPES =   32772;
	public static final int TYPE_ID_USERS =          32773;
	public static final int TYPE_ID_GROUPS =         32774;
	public static final int TYPE_ID_SYSTEM =         32775;

	public static final int SECURITY_ADD_CALENDARS = 2;
	public static final int SECURITY_EDIT_USER = 8;
	public static final int SECURITY_ADD_USER = 16;
	public static final int SECURITY_SYSCONFIG = 64;
	public static final int SECURITY_ITEMTYPES = 128;
	public static final int SECURITY_RESOURCES = 256;
	public static final int SECURITY_RESOURCE_TYPES = 512;
	public static final int SECURITY_PURGEDB = 2048;
	public static final int SECURITY_STYLESETUP = 4096;
	public static final int SECURITY_ADVANCED_EDIT = 8192;

	private Properties pWebApp = null;
	private MHdbConn mhdb = null;
	private String preFile = null;
	private String postFile = null;

	private AccessSchema250(String configPropFile, String preFilePath) {
		preFile = preFilePath;
		
		// Set configuration properties
		pWebApp = WebApp.getProperties();

		FileInputStream confIn = null;
		String sKey = null;
		Properties p = new Properties();
		boolean success = false;

		try {
			confIn = new FileInputStream(configPropFile);
			p.load(confIn);
			success = true;
		} catch ( FileNotFoundException fne ) {
			logFatalError("Specified configuration file not found: " +
						  configPropFile.toString() + "!", fne);
		} catch ( IOException ioe ) {
			logFatalError("IO Error reading specified configuration file: " +
						  configPropFile.toString() + "!", ioe);
		}

		if ( !success ) {
			return;
		}

		Enumeration e = p.propertyNames();

		while ( e.hasMoreElements() ) {
			sKey = (String)e.nextElement();
			pWebApp.setProperty(sKey, p.getProperty(sKey));
		}

		try {
			confIn.close();
		} catch ( IOException ioe ) {
			logNonFatalError("Non-fatal IO Error closing configuration properties file: " + ioe.toString());
		}
	}

	private static void logNonFatalError(String c) {
		System.out.println(PROC_NAME);
		System.out.println(ERR_PREFIX);
		System.out.println(c);
	}

	private static void logFatalError(String c, Exception e) {
		System.out.println(PROC_NAME);
		System.out.println(ERR_PREFIX_FATAL);
		System.out.println(c);
		e.printStackTrace(System.out);
	}

	private void execUpdate(String cmd) throws SQLException {
		SQLException seThrow=null;
		Statement s=getSqlStmt();
		try {
			s.executeUpdate(cmd);	
		} catch (SQLException e) {
			// TODO: Implement Exception Handling
			seThrow=e;
			s.getConnection().rollback();
		}
		if (seThrow!=null) {
			throw seThrow;
		}
		
		
	}

	private ResultSet execSelect(String cmd) throws SQLException {
		SQLException seThrow=null;
		ResultSet rs=null;
		try {
		rs=getSqlStmt().executeQuery(cmd);	
		} catch (SQLException e) {
			seThrow=e;
			// TODO: Implement Exception Handling
			logFatalError("Executing Select ["+cmd+"]",e);
		}
		if (seThrow!=null) {
			throw seThrow;
		}
		return rs;
		
		
	}

	private Statement getSqlStmt() throws SQLException {
		if ( mhdb == null ) {
			MHdbQueue dbQueue = MHdbPool.getInstance().getQueue(
															   pWebApp.getProperty(WebApp.JDBC_DRIVER),
															   pWebApp.getProperty(WebApp.JDBC_CONNECT_STRING),
															   pWebApp.getProperty(WebApp.JDBC_USER_ID, ""),
															   pWebApp.getProperty(WebApp.JDBC_PASSWORD, "")
															   );

			mhdb = dbQueue.getConnection();
		}

		return mhdb.cn.createStatement();
	}

	private boolean processSqlFile(String filePath) {
		boolean success = true;
		System.out.println("Processing "+filePath);
		StringBuffer sbSql = new StringBuffer();
		BufferedReader sqlIn = null;
		String nextLine;

		try {
			sqlIn = new BufferedReader(new FileReader(filePath));

			while ( (nextLine = sqlIn.readLine()) != null ) {
				if ( nextLine.length() > 0 ) {
					if ( nextLine.equals(CMD_TERM_CHAR) && (sbSql.length() > 0) ) {
						try {
							execUpdate(sbSql.toString());   
						} catch (SQLException seSchema) {

							// This might throw an error on a view that doesn't exist...
							logNonFatalError("Non-fatal: " + seSchema.toString());
							logNonFatalError(sbSql.toString());
						}

						sbSql.setLength(0);
					} else {
						sbSql.append(" ").append(nextLine);
					}
				}
			}
		} catch ( IOException ioe ) {
			success = false;
			logFatalError("IO Error reading SQL command file!", ioe);
		}
		if ( sqlIn != null ) {
			try {
				sqlIn.close();
			} catch ( IOException ioe ) {
				logNonFatalError("Non-fatal IO Error closing configuration properties file: " + ioe.toString());
			}
		}

		return success;
	}

	private boolean setStep(int stepNo) {
		boolean success = true;

		try {
			execUpdate("INSERT INTO convert250 (currStep) VALUES (" + Integer.toString(stepNo) + ")");
		} catch ( SQLException se ) {
			logFatalError("Internal SQL Error setting progress step!", se);
			success = false;
		}

		return success;
	}

	private Vector getCandInstTokens(int objTypeId) throws SQLException {
		Vector v = new Vector();

		ResultSet rs = execSelect(
								 "SELECT token FROM candidatetokens WHERE object_type_id=" +
								 Integer.toString(objTypeId) + " AND instance=1"
								 );

		while ( rs.next() ) {
			v.add(rs.getString("token"));
		}

		return v;
	}

	private Vector getUserDefaultTokens(int objTypeId) throws SQLException {
		Vector v = new Vector();

		ResultSet rs = execSelect(
								 "SELECT token FROM candidatetokens WHERE object_type_id=" +
								 Integer.toString(objTypeId) + " AND instance=1 AND is_default=1"
								 );

		while ( rs.next() ) {
			v.add(rs.getString("token"));
		}

		rs.close();

		return v;
	}

	private Vector getOwnerDefaultTokens(int objTypeId) throws SQLException {
		Vector v = new Vector();

		ResultSet rs = execSelect(
								 "SELECT token FROM candidatetokens WHERE object_type_id=" +
								 Integer.toString(objTypeId) + " AND instance=1 AND creator_default=1"
								 );

		while ( rs.next() ) {
			v.add(rs.getString("token"));
		}

		rs.close();

		return v;
	}

	private void cleanup()  {
		if ( mhdb != null ) {
			mhdb.freeConnection();
		}
	}

	/*
	 * arg 1: configuration.properties
	 * arg 2: 250 Inserts.sql
	 *
	 */
	public static void main(String[] args) {
		ResultSet rs,
			rs2;
		Vector vObjs,
			vTokens;
		Enumeration eObjs,
			eTokens;
		int nextMask,
			nextId;
		objectsecurity os;

		AccessSchema250 sch = new AccessSchema250(args[0], args[1]);

		// Check or initialize the conversion step table
		int onStep = 0;

		try {
			ResultSet rsStep = sch.execSelect("SELECT max(currstep) as CurrStep FROM convert250");
			rsStep.next();
			try {
				onStep = rsStep.getInt("currstep");
				rsStep.next();
				rsStep.close();
			} catch ( SQLException se2 ) {
				logNonFatalError("Non-fatal SQL Error closing configuration properties file: " + se2.toString());
			}
		} catch ( SQLException se ) {
			// Table doesn't yet exist, so treat this as the first attempt
			try {
				sch.execUpdate("CREATE TABLE convert250 (currstep int)");
			} catch ( SQLException se3 ) {
				logFatalError("SQL Error creating conversion step table!", se3);
				return;
			}

			if ( !sch.setStep(0) ) {
				return;
			}
		}


		//---------- Pre-conversion SQL

		if ( onStep == 0 ) {
			onStep++;
			if ( sch.processSqlFile(sch.preFile) ) {
				if ( !sch.setStep(onStep) ) {
					return;
				}
			} else {
				return;
			}
		}
		// CalItem exception conversion
		if ( onStep == 1 ) {
			Statement sOuter, sInner;
			ResultSet rsOuter, rsInner;
			MHdbConn dbc=WebApp.getConnection(),dbc2=WebApp.getConnection();
			PreparedStatement ps;
			try {


				sOuter=dbc.cn.createStatement();
				rsOuter=sOuter.executeQuery("Select distinct(cal_item_id) as cal_item_id_a from calItemException");
				ps=dbc2.cn.prepareStatement("Update calitem set Exceptions=? where cal_item_id=?");

				while (rsOuter.next()) {
					int iOuter=rsOuter.getInt("cal_item_id_a");
					StringBuffer sbData=new StringBuffer(256);
					sInner=dbc.cn.createStatement();
					rsInner=sInner.executeQuery("select ddate from calitemexception where cal_item_id="+iOuter+" order by ddate");
					while (rsInner.next()) {
						sbData.append(rsInner.getInt("ddate")).append('\n');
					}
					rsInner.close();
					ps.setString(1,sbData.toString());
					ps.setInt(2,iOuter);
					ps.executeUpdate();
				}
				rsOuter.close();
				ps.close();
				dbc.freeConnection();
				dbc2.freeConnection();
			} catch (SQLException seExceptions) {
				logFatalError("converting Exceptions",seExceptions);
				return;
			}
			onStep++;
			if ( !sch.setStep(onStep) ) {
				return;
			}
		}

		// Add guest user
		if ( onStep == 2 ) {
			try {
				sch.execUpdate("INSERT INTO USERS (user_name, password, full_name, security_level,enabled) VALUES ('GUEST', 'DISABLED', 'Guest User for Public Calendars', 0,0)");
				onStep++;
				if ( !sch.setStep(onStep) ) {
					return;
				}
			} catch ( SQLException se ) {
				logFatalError("SQL Error adding guest user record!", se);
				return;
			}
		}

		// Add "Everyone" group
		groups g = null;

		if ( onStep == 3 ) {
			g = new groups();
			g.group_name = "Everyone";

			try {
				g.update();
				onStep++;
				if ( !sch.setStep(onStep) ) {
					return;
				}

			} catch ( SQLException se ) {

				logFatalError("SQL Error adding Everyone group!", se);
				return;
			}
		} else {
			g = groups.getGroupByName("Everyone");
		}

		// Cache a list of all user_id/security_level records, as we'll need to iterate it several times
		Hashtable htUsers = new Hashtable();
		Enumeration eKeys;
		Integer oInt;

		try {
			rs = sch.execSelect("SELECT user_id, security_level FROM Users");

			while ( rs.next() ) {
				htUsers.put(Integer.valueOf(rs.getInt("user_id")), Integer.valueOf(rs.getInt("security_level")));
			}

			rs.close();
		} catch ( SQLException se ) {
			logFatalError("SQL Error retrieving full user list!", se);
			return;
		}

		// Add all users to "Everyone" group
		if ( onStep == 4 ) {
			groupusers gu;
			eKeys = htUsers.keys();

			try {
				while ( eKeys.hasMoreElements() ) {
					gu = new groupusers();
					gu.group_id = g.group_id;
					int iUserID=((Integer)eKeys.nextElement()).intValue();

					gu.user_id = iUserID;
					System.out.println("Group ID="+g.group_id+", User ID="+iUserID);
					gu.update();	
				}
				onStep++;
				if ( !sch.setStep(onStep) ) {
					return;
				}
			} catch ( SQLException se ) {
				logFatalError("SQL Error adding guest user to group Everyone!", se);
				return;
			}
		}

		// System permissions, and also non-calendar instance permissions
		//
		// (Note: These could all be done in a single pass on the user table, but
		//        since performance is not a consideration here, splitting all of it
		//        into multiple passes gives us finer granularity on step separation.)

		// Resource types
		Vector vUserTokens;
		Enumeration eUserTokens;
		if ( onStep == 5 ) {
			vObjs = resourcetype.getItems();
			logNonFatalError("vObjs.size()=="+vObjs.size());
			if ( vObjs.isEmpty() ) {
				logNonFatalError("No Resource Types found!");
			} else {
				try {

					resourcetype rt;
					vUserTokens=sch.getUserDefaultTokens(TYPE_ID_RESOURCETYPE);

					eObjs=vObjs.elements();
					while (eObjs.hasMoreElements()) {
						rt=(resourcetype)eObjs.nextElement();
						eUserTokens=vUserTokens.elements();                 
						while (eUserTokens.hasMoreElements()) {
							os=new objectsecurity();
							os.entity_id = g.group_id;
							os.entity_type=objectsecurity.ENTITY_TYPE_GROUP;
							os.secured_obj_id = rt.resource_type_id;
							os.secured_obj_type = TYPE_ID_RESOURCETYPE;
							String sToken=(String)eUserTokens.nextElement();
							System.out.println("Secured_Obj_ID="+rt.resource_type_id+", Entity_ID="+g.group_id+", Token="+sToken);
							os.token=sToken;
							os.update();
						}
					}

					vTokens = sch.getCandInstTokens(TYPE_ID_RESOURCETYPE);
					eKeys = htUsers.keys();
					while ( eKeys.hasMoreElements() ) {
						oInt = (Integer)eKeys.nextElement();
						nextMask = ((Integer)htUsers.get(oInt)).intValue();

						if ( ((nextMask & SECURITY_RESOURCE_TYPES) == SECURITY_RESOURCE_TYPES) ) {
							nextId = oInt.intValue();

							// Give non-instance "Add" token
							os = new objectsecurity();
							os.entity_id = nextId;
							os.entity_type = objectsecurity.ENTITY_TYPE_USER;
							os.secured_obj_type = TYPE_ID_RESOURCETYPE;
							os.secured_obj_id = 0;
							os.token = candidatetokens.TOKEN_ADD;
							os.update();

							// Give all instance tokens, all objects
							eObjs = vObjs.elements();


							while ( eObjs.hasMoreElements() ) {
								rt=(resourcetype)eObjs.nextElement();
								eTokens = vTokens.elements();
								while ( eTokens.hasMoreElements() ) {
									os = new objectsecurity();
									os.entity_id = nextId;
									os.entity_type = objectsecurity.ENTITY_TYPE_USER;
									os.secured_obj_type = TYPE_ID_RESOURCETYPE;
									os.secured_obj_id = rt.resource_type_id;
									String sToken=(String)eTokens.nextElement();
									System.out.println("Entity ID="+nextId+", secured_obj_id="+rt.resource_type_id+",token="+sToken);
									os.token = sToken;
									os.update();
								}
							}
						}
					}

				} catch ( SQLException se ) {
					logFatalError("SQL Error converting Resource Type privileges!", se);
					return;
				}
			}
			onStep++;
			if ( !sch.setStep(onStep) ) {
				return;
			}
		}

		// Resources
		if ( onStep == 6 ) {
			vObjs = resource.getItems();

			if ( vObjs.size() > 0 ) {
				try {
					resource r;
					vUserTokens=sch.getUserDefaultTokens(TYPE_ID_RESOURCES);
					eObjs=vObjs.elements();
					while (eObjs.hasMoreElements()) {
						r=(resource)eObjs.nextElement();
						eUserTokens=vUserTokens.elements();                 
						while (eUserTokens.hasMoreElements()) {
							os=new objectsecurity();
							os.entity_id = g.group_id;
							os.entity_type=objectsecurity.ENTITY_TYPE_GROUP;
							os.secured_obj_id = r.resource_id;
							os.secured_obj_type = TYPE_ID_RESOURCES;
							os.token=(String)eUserTokens.nextElement();
							os.update();
						}
					}

					vTokens = sch.getCandInstTokens(TYPE_ID_RESOURCES);
					eKeys = htUsers.keys();
					while ( eKeys.hasMoreElements() ) {
						oInt = (Integer)eKeys.nextElement();
						nextMask = ((Integer)htUsers.get(oInt)).intValue();

						if ( ((nextMask & SECURITY_RESOURCES) == SECURITY_RESOURCES ) ) {
							nextId = oInt.intValue();

							// Give non-instance "Add" token
							os = new objectsecurity();
							os.entity_id = nextId;
							os.entity_type = objectsecurity.ENTITY_TYPE_USER;
							os.secured_obj_type = TYPE_ID_RESOURCES;
							os.secured_obj_id = 0;
							os.token = candidatetokens.TOKEN_ADD;
							os.update();

							// Give all instance tokens, all objects
							eObjs = vObjs.elements();


							while ( eObjs.hasMoreElements() ) {
								r=(resource)eObjs.nextElement();
								eTokens = vTokens.elements();
								while ( eTokens.hasMoreElements() ) {
									os = new objectsecurity();
									os.entity_id = nextId;
									os.entity_type = objectsecurity.ENTITY_TYPE_USER;
									os.secured_obj_type = TYPE_ID_RESOURCES;
									os.secured_obj_id = r.resource_id;
									os.token = (String)eTokens.nextElement();
									os.update();
								}
							}
						}
					}

				} catch ( SQLException se ) {
					logFatalError("SQL Error converting Resource privileges!", se);
					return;
				}
			}
			onStep++;
			if ( !sch.setStep(onStep) ) {
				return;
			}
		}

		// Item types
		if ( onStep == 7 ) {
			vObjs = calitemtypes.getItems();

			if ( vObjs.size() > 0 ) {
				try {
					calitemtypes cit;
					vUserTokens=sch.getUserDefaultTokens(TYPE_ID_CALITEMTYPES);

					eObjs=vObjs.elements();
					while (eObjs.hasMoreElements()) {
						cit=(calitemtypes)eObjs.nextElement();
						eUserTokens=vUserTokens.elements();                 
						while (eUserTokens.hasMoreElements()) {
							os=new objectsecurity();
							os.entity_id = g.group_id;
							os.entity_type=objectsecurity.ENTITY_TYPE_GROUP;
							os.secured_obj_id = cit.item_type_id;
							os.secured_obj_type = TYPE_ID_CALITEMTYPES;
							os.token=(String)eUserTokens.nextElement();
							os.update();
						}
					}

					vTokens = sch.getCandInstTokens(TYPE_ID_CALITEMTYPES);
					eKeys = htUsers.keys();

					while ( eKeys.hasMoreElements() ) {
						oInt = (Integer)eKeys.nextElement();
						nextMask = ((Integer)htUsers.get(oInt)).intValue();

						if ( ((nextMask & SECURITY_ITEMTYPES) == SECURITY_ITEMTYPES ) ) {
							nextId = oInt.intValue();

							// Give non-instance "Add" token
							os = new objectsecurity();
							os.entity_id = nextId;
							os.entity_type = objectsecurity.ENTITY_TYPE_USER;
							os.secured_obj_type = TYPE_ID_CALITEMTYPES;
							os.secured_obj_id = 0;
							os.token = candidatetokens.TOKEN_ADD;
							os.update();

							// Give all instance tokens, all objects
							eObjs = vObjs.elements();


							while ( eObjs.hasMoreElements() ) {
								cit=(calitemtypes)eObjs.nextElement();
								eTokens = vTokens.elements();
								while ( eTokens.hasMoreElements() ) {
									os = new objectsecurity();
									os.entity_id = nextId;
									os.entity_type = objectsecurity.ENTITY_TYPE_USER;
									os.secured_obj_type = TYPE_ID_CALITEMTYPES;
									os.secured_obj_id = cit.item_type_id;
									os.token = (String)eTokens.nextElement();
									os.update();
								}
							}
						}
					}

				} catch ( SQLException se ) {
					logFatalError("SQL Error converting Item Type privileges!", se);
					return;
				}
			}
			onStep++;
			if ( !sch.setStep(onStep) ) {
				return;
			}
		}

		// User security
		if ( onStep == 8 ) {
			eKeys = htUsers.keys();

			try {
				while ( eKeys.hasMoreElements() ) {
					oInt = (Integer)eKeys.nextElement();
					nextId = oInt.intValue();
					nextMask = ((Integer)htUsers.get(oInt)).intValue();

					if ( ((nextMask & SECURITY_EDIT_USER) == SECURITY_EDIT_USER ) ) {
						os = new objectsecurity();
						os.entity_id = nextId;
						os.entity_type = objectsecurity.ENTITY_TYPE_USER;
						os.secured_obj_type = TYPE_ID_USERS;
						os.secured_obj_id = 0;
						os.token = candidatetokens.TOKEN_EDIT;
						os.update();

						os = new objectsecurity();
						os.entity_id = nextId;
						os.entity_type = objectsecurity.ENTITY_TYPE_USER;
						os.secured_obj_type = TYPE_ID_USERS;
						os.secured_obj_id = 0;
						os.token = candidatetokens.TOKEN_DELETE;
						os.update();

						os = new objectsecurity();
						os.entity_id = nextId;
						os.entity_type = objectsecurity.ENTITY_TYPE_USER;
						os.secured_obj_type = TYPE_ID_GROUPS;
						os.secured_obj_id = 0;
						os.token = candidatetokens.TOKEN_EDIT;
						os.update();

						os = new objectsecurity();
						os.entity_id = nextId;
						os.entity_type = objectsecurity.ENTITY_TYPE_USER;
						os.secured_obj_type = TYPE_ID_GROUPS;
						os.secured_obj_id = 0;
						os.token = candidatetokens.TOKEN_DELETE;
						os.update();

						os = new objectsecurity();
						os.entity_id = nextId;
						os.entity_type = objectsecurity.ENTITY_TYPE_USER;
						os.secured_obj_type = TYPE_ID_SYSTEM;
						os.secured_obj_id = 0;
						os.token = candidatetokens.TOKEN_PERMISSION_FIXUP;
						os.update();
					}

					if ( ((nextMask & SECURITY_ADD_USER) == SECURITY_ADD_USER ) ) {
						os = new objectsecurity();
						os.entity_id = nextId;
						os.entity_type = objectsecurity.ENTITY_TYPE_USER;
						os.secured_obj_type = TYPE_ID_SYSTEM;
						os.secured_obj_id = 0;
						os.token = candidatetokens.TOKEN_ADD;
						os.update();

						os = new objectsecurity();
						os.entity_id = nextId;
						os.entity_type = objectsecurity.ENTITY_TYPE_USER;
						os.secured_obj_type = TYPE_ID_GROUPS;
						os.secured_obj_id = 0;
						os.token = candidatetokens.TOKEN_ADD;
						os.update();
					}
				}
				onStep++;
				if ( !sch.setStep(onStep) ) {
					return;
				}
			} catch ( SQLException se ) {
				logFatalError("SQL Error converting User Security privileges!", se);
				return;
			}
		}

		// Misc system permissions
		if ( onStep == 9 ) {
			eKeys = htUsers.keys();

			try {
				while ( eKeys.hasMoreElements() ) {
					oInt = (Integer)eKeys.nextElement();
					nextId = oInt.intValue();
					nextMask = ((Integer)htUsers.get(oInt)).intValue();

					if ( ((nextMask & SECURITY_SYSCONFIG) == SECURITY_SYSCONFIG) ) {
						os = new objectsecurity();
						os.entity_id = nextId;
						os.entity_type = objectsecurity.ENTITY_TYPE_USER;
						os.secured_obj_type = TYPE_ID_SYSTEM;
						os.secured_obj_id = 0;
						os.token = candidatetokens.TOKEN_EDIT_CONFIG;
						os.update();
					}

					if ( ((nextMask & SECURITY_PURGEDB) == SECURITY_PURGEDB) ) {
						os = new objectsecurity();
						os.entity_id = nextId;
						os.entity_type = objectsecurity.ENTITY_TYPE_USER;
						os.secured_obj_type = TYPE_ID_SYSTEM;
						os.secured_obj_id = 0;
						os.token = candidatetokens.TOKEN_PURGE_DB;
						os.update();
					}

					if ( ((nextMask & SECURITY_STYLESETUP) == SECURITY_STYLESETUP) ) {
						os = new objectsecurity();
						os.entity_id = nextId;
						os.entity_type = objectsecurity.ENTITY_TYPE_USER;
						os.secured_obj_type = TYPE_ID_SYSTEM;
						os.secured_obj_id = 0;
						os.token = candidatetokens.TOKEN_MANAGE_ITEM_STYLES;
						os.update();
					}

					if ( ((nextMask & SECURITY_ADVANCED_EDIT) == SECURITY_ADVANCED_EDIT) ) {
						os = new objectsecurity();
						os.entity_id = nextId;
						os.entity_type = objectsecurity.ENTITY_TYPE_USER;
						os.secured_obj_type = TYPE_ID_SYSTEM;
						os.secured_obj_id = 0;
						os.token = candidatetokens.TOKEN_BULK_EXCEPTION;
						os.update();

						os = new objectsecurity();
						os.entity_id = nextId;
						os.entity_type = objectsecurity.ENTITY_TYPE_USER;
						os.secured_obj_type = TYPE_ID_SYSTEM;
						os.secured_obj_id = 0;
						os.token = candidatetokens.TOKEN_HOLIDAY_ADD;
						os.update();
					}

					if ( ((nextMask & SECURITY_ADD_CALENDARS) == SECURITY_ADD_CALENDARS) ) {
						os = new objectsecurity();
						os.entity_id = nextId;
						os.entity_type = objectsecurity.ENTITY_TYPE_USER;
						os.secured_obj_type = TYPE_ID_CALENDAR;
						os.secured_obj_id = 0;
						os.token = candidatetokens.TOKEN_ADD;
						os.update();

						os = new objectsecurity();
						os.entity_id = nextId;
						os.entity_type = objectsecurity.ENTITY_TYPE_USER;
						os.secured_obj_type = TYPE_ID_CALGROUP;
						os.secured_obj_id = 0;
						os.token = candidatetokens.TOKEN_ADD;
						os.update();
					}
				}

				onStep++;
				if ( !sch.setStep(onStep) ) {
					return;
				}
			} catch ( SQLException se ) {
				logFatalError("SQL Error converting misc. System privileges!", se);
				return;
			}
		}

		// CalGroup permissions
		Vector vUserDefToke,
			vOwnerDefToke;
		Enumeration eUserDefToke,eOwnerDefToke;
		boolean bShowAll=WebApp.getBooleanProperty("ShowAllCalendars");
		groups gu=null;	// If show all calendars is set to yes, create an additional users group and give them view to every
						// calendar.
		if (bShowAll) {
			gu=new groups();
			gu.group_name="Users";
			gu.descr="Group for pre-2.5 ShowAllCalendars permission.";
			try {
				gu.update();    
				groupusers gusers;
				rs = sch.execSelect("SELECT user_id FROM Users where Enabled is null or enabled>0");

				while ( rs.next()) {
					gusers = new groupusers();
					gusers.group_id = gu.group_id;
					gusers.user_id = rs.getInt("user_id");
					gusers.update();
				}
				rs.close();
			} catch (SQLException seUsers) {
				logFatalError("Error adding group users.",seUsers);
			}

		}
		if ( onStep == 10 ) {
			try {
				vOwnerDefToke=sch.getOwnerDefaultTokens(TYPE_ID_CALGROUP);
				vUserDefToke = sch.getUserDefaultTokens(TYPE_ID_CALGROUP);
				rs = sch.execSelect("SELECT * FROM calgroup");

				while ( rs.next() ) {
					nextId = rs.getInt("cal_group_id");
					int iOwnerID=rs.getInt("owner");
					if ( rs.getInt("published") > 0 ) {
						os = new objectsecurity();
						os.entity_id = g.group_id;
						os.entity_type = objectsecurity.ENTITY_TYPE_GROUP;
						os.secured_obj_type = TYPE_ID_CALGROUP;
						os.secured_obj_id = nextId;
						os.token = candidatetokens.TOKEN_VIEW;
						os.update();
					} else {
						if (bShowAll) {
							os = new objectsecurity();
							os.entity_id = gu.group_id;
							os.entity_type = objectsecurity.ENTITY_TYPE_GROUP;
							os.secured_obj_type = TYPE_ID_CALGROUP;
							os.secured_obj_id = nextId;
							os.token = candidatetokens.TOKEN_VIEW;
							os.update();
						}
					}
					eOwnerDefToke = vOwnerDefToke.elements();
					while ( eOwnerDefToke.hasMoreElements() ) {
						os = new objectsecurity();
						os.entity_id = iOwnerID;
						os.entity_type = objectsecurity.ENTITY_TYPE_USER;
						os.secured_obj_type = TYPE_ID_CALGROUP;
						os.secured_obj_id = nextId;
						os.token = (String)eOwnerDefToke.nextElement();
						try {
							os.update();    
						} catch (SQLException seUnused2) {
							// We don't care.
						}

					}

					rs2 = sch.execSelect("SELECT * FROM calgroupuser WHERE cal_group_id=" +
										 Integer.toString(nextId)+"and user_id<>"+iOwnerID);

					while ( rs2.next() ) {
						eUserDefToke = vUserDefToke.elements();
						while ( eUserDefToke.hasMoreElements() ) {
							os = new objectsecurity();
							os.entity_id = rs2.getInt("user_id");
							os.entity_type = objectsecurity.ENTITY_TYPE_USER;
							os.secured_obj_type = TYPE_ID_CALGROUP;
							os.secured_obj_id = nextId;
							os.token = (String)eUserDefToke.nextElement();
							os.update();    

						}
					}

					rs2.close();
				}

				rs.close();
				onStep++;
				if ( !sch.setStep(onStep) ) {
					return;
				}
			} catch ( SQLException se ) {
				logFatalError("SQL Error converting Calendar Group privileges!", se);
				return;
			}
		}

		// Calendar permissions
		int ownerId;

		if ( onStep == 11 ) {
			try {
				vUserDefToke = sch.getUserDefaultTokens(TYPE_ID_CALENDAR);
				vOwnerDefToke = sch.getOwnerDefaultTokens(TYPE_ID_CALENDAR);

				rs = sch.execSelect("SELECT * FROM calendar");

				while ( rs.next() ) {
					nextId = rs.getInt("calendar_id");
					ownerId = rs.getInt("owner");

					if ( rs.getInt("published") > 0 ) {
						os = new objectsecurity();
						os.entity_id = g.group_id;
						os.entity_type = objectsecurity.ENTITY_TYPE_GROUP;
						os.secured_obj_type = TYPE_ID_CALENDAR;
						os.secured_obj_id = nextId;
						os.token = candidatetokens.TOKEN_VIEW;
						os.update();
					} else {
						if (bShowAll) {
							os = new objectsecurity();
							os.entity_id = gu.group_id;
							os.entity_type = objectsecurity.ENTITY_TYPE_GROUP;
							os.secured_obj_type = TYPE_ID_CALENDAR;
							os.secured_obj_id = nextId;
							os.token = candidatetokens.TOKEN_VIEW;
							os.update();
						}
					}

					eOwnerDefToke = vOwnerDefToke.elements();
					while ( eOwnerDefToke.hasMoreElements() ) {
						os = new objectsecurity();
						os.entity_id = ownerId;
						os.entity_type = objectsecurity.ENTITY_TYPE_USER;
						os.secured_obj_type = TYPE_ID_CALENDAR;
						os.secured_obj_id = nextId;

						os.token = (String)eOwnerDefToke.nextElement();
						os.update();    
					}
					rs2 = sch.execSelect("SELECT user_id FROM caluser WHERE calendar_id=" +
										 Integer.toString(nextId)+" and user_id<>"+ownerId);

					while ( rs2.next() ) {
						eUserDefToke = vUserDefToke.elements();
						int iUserID=rs2.getInt("user_id");
						while ( eUserDefToke.hasMoreElements() ) {
							os = new objectsecurity();
							
							os.entity_id = iUserID;
							os.entity_type = objectsecurity.ENTITY_TYPE_USER;
							os.secured_obj_type = TYPE_ID_CALENDAR;
							os.secured_obj_id = nextId;
							os.token = (String)eUserDefToke.nextElement();
							os.update();    

						}

					}

					rs2.close();
				}

				rs.close();
				onStep++;
				if ( !sch.setStep(onStep) ) {
					return;
				}
			} catch ( SQLException se ) {
				logFatalError("SQL Error converting Calendar privileges!", se);
				return;
			}
		}

    	//---------- Cleanup

		try {
			if ( onStep == 12 ) {
				sch.execUpdate("DROP TABLE convert250");
			}

			sch.cleanup();
		} catch ( SQLException se ) {
			logFatalError("SQL Error dropping conversion step table!", se);
		}
	}
}
