connect 'jdbc:derby:calendardb;create=true';
create table cd_dbversion (
 version_str VARCHAR(16) NOT NULL PRIMARY KEY
 );
CREATE TABLE groups (
 group_id int NOT NULL PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY,
 group_name VARCHAR(64) NOT NULL,
 descr CLOB ,
 ldap_equivs CLOB ,
 CONSTRAINT UQ_G_GN UNIQUE (group_name)
 );
CREATE TABLE users (
 user_id int NOT NULL PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY,
 user_name VARCHAR(64) NOT NULL,
 password VARCHAR(64) NOT NULL,
 full_name VARCHAR(64),
 email_address VARCHAR(128),
 sms_number VARCHAR(64) ,
 enabled int DEFAULT 1 NOT NULL,
 last_signin timestamp ,
 default_calendar int ,
 last_ip_address VARCHAR(128),
 last_browser VARCHAR(80),
 blogincount int DEFAULT 0 NOT NULL,
 lastblogin timestamp ,
 pwchangedate timestamp DEFAULT CURRENT_TIMESTAMP NOT NULL,
 pwhistory CLOB ,
 user_prefs CLOB ,
 prefs_flag int DEFAULT 16655,
 grlogrem int DEFAULT 3,
 tz_string VARCHAR(64) ,
 locales VARCHAR(80) ,
 plannerstart VARCHAR(32) ,
 plannerend VARCHAR(32) ,
 CONSTRAINT UQ_User_UserName UNIQUE (User_Name)
 );
CREATE TABLE groupusers (
 group_id int NOT NULL,
 user_id int NOT NULL,
 CONSTRAINT PK_GroupUsers PRIMARY KEY (group_id, user_id),
 CONSTRAINT FK_GU_G FOREIGN KEY (group_id) References groups (group_id) ON DELETE CASCADE,
 CONSTRAINT FK_GU_U FOREIGN KEY (user_id) References users (user_id) ON DELETE CASCADE
 );
CREATE TABLE objecttypes (
 object_type_id int NOT NULL PRIMARY KEY,
 type_name VARCHAR(32) NOT NULL,
 sort_order int NOT NULL,
 table_name VARCHAR(32) ,
 field_name VARCHAR(32) ,
 CONSTRAINT UQ_OT_TN UNIQUE (type_name)
 );
CREATE TABLE objectsecurity (
 entity_id int NOT NULL,
 entity_type int NOT NULL,
 secured_obj_type int NOT NULL,
 secured_obj_id int DEFAULT 0 NOT NULL,
 token VARCHAR(64) NOT NULL,
 CONSTRAINT PK_ObjSec PRIMARY KEY (entity_id, entity_type, secured_obj_type, secured_obj_id, token),
 CONSTRAINT FK_OS_OT FOREIGN KEY (secured_obj_type) References objecttypes (object_type_id)
 );
CREATE INDEX IX_OS_SO_EI ON objectsecurity (secured_obj_id, secured_obj_type, entity_id, entity_type);
CREATE TABLE candidatetokens (
 object_type_id int NOT NULL,
 instance int DEFAULT 0 NOT NULL ,
 token VARCHAR(64) NOT NULL,
 sort_order int NOT NULL,
 is_default int DEFAULT 0 NOT NULL,
 creator_default int DEFAULT 1 NOT NULL,
 CONSTRAINT FK_CT_OT FOREIGN KEY (object_type_id) References objecttypes (object_type_id),
 CONSTRAINT PK_CT PRIMARY KEY (object_type_id, token)
 );
CREATE TABLE calendar (
 calendar_id int NOT NULL PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY,
 name VARCHAR(64) NOT NULL,
 short_name VARCHAR(12) ,
 default_style int DEFAULT 0 NOT NULL,
 default_type int DEFAULT 0 NOT NULL,
 def_privacy int DEFAULT 0 NOT NULL,
 style_sheet VARCHAR(128) ,
 image_name VARCHAR(64) ,
 prefs int DEFAULT 0 NOT NULL,
 header_file VARCHAR(128) ,
 footer_file VARCHAR(128) ,
 last_edited timestamp ,
 last_edit_by VARCHAR(64) ,
 long_description CLOB ,
 CONSTRAINT UQ_Calendar_Name UNIQUE (name)
 );
CREATE TABLE calguestmap (
 mapentry_id int NOT NULL PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY,
 strexpr VARCHAR(128) NOT NULL,
 guest_user int NOT NULL,
 CONSTRAINT FK_CGM_U FOREIGN KEY (guest_user) references users (user_id)
 );
CREATE TABLE calimportlinks (
 callink_id int NOT NULL PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY,
 calendar_id int NOT NULL,
 refresh_interval int NOT NULL,
 enabled int DEFAULT 0 NOT NULL,
 last_executed timestamp NOT NULL,
 last_messages CLOB ,
 settings CLOB ,
 CONSTRAINT FK_CIL_CAL FOREIGN KEY (calendar_id) references calendar (calendar_id) ON DELETE CASCADE
 );
CREATE TABLE calitemtypes (
 item_type_id int NOT NULL PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY,
 description VARCHAR(64) NOT NULL,
 CONSTRAINT UQ_CalItemTypes_Desc UNIQUE (Description)
 );
CREATE TABLE location (
 location_id int NOT NULL PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY,
 parent_location int ,
 location_name VARCHAR(128) NOT NULL,
 location_address VARCHAR(255) ,
 latitude DOUBLE ,
 longitude DOUBLE ,
 flags int DEFAULT 0 NOT NULL,
 location_comments CLOB ,
 CONSTRAINT FK_LOC_LOC FOREIGN KEY (parent_location) references location (location_id),
 CONSTRAINT UQ_LOC_NAME UNIQUE (location_name)
 );
CREATE TABLE calitem (
 cal_item_id int NOT NULL PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY,
 calendar_id int NOT NULL,
 item_type_id int NOT NULL,
 cancelled boolean DEFAULT false NOT NULL,
 month int DEFAULT 0 NOT NULL ,
 day int DEFAULT 0 NOT NULL,
 dow int DEFAULT 0 NOT NULL,
 recurs int DEFAULT 0 NOT NULL,
 duration smallint DEFAULT 0 NOT NULL,
 rinterval smallint DEFAULT 0 NOT NULL,
 starting_date int ,
 ending_date int ,
 style_id int DEFAULT 0 NOT NULL,
 parent_item int DEFAULT 0 NOT NULL,
 link_type int DEFAULT 0 NOT NULL,
 owner int ,
 privacy_opts int DEFAULT 0 NOT NULL,
 approved int DEFAULT 0 NOT NULL,
 starting_time VARCHAR(12),
 ending_time VARCHAR(12),
 rec_type smallint DEFAULT 0 NOT NULL,
 location_id int ,
 description VARCHAR(80) NOT NULL,
 contact_name VARCHAR(128),
 contact_info VARCHAR(128),
 add_info_url VARCHAR(128),
 itemguid VARCHAR(128) NOT NULL,
 seq_value int DEFAULT 0 NOT NULL,
 tz_string VARCHAR(64) ,
 last_update timestamp NOT NULL,
 exceptions CLOB ,
 long_description CLOB ,
 xml_data CLOB ,
 CONSTRAINT FK_CalItem_Calendar FOREIGN KEY (Calendar_id) References calendar (calendar_id) ON DELETE CASCADE,
 CONSTRAINT FK_CalItem_CalItemType FOREIGN KEY (Item_Type_id) References calitemtypes (item_type_id),
 CONSTRAINT FK_CI_LOC FOREIGN KEY (location_id) references location (location_id),
 CONSTRAINT FK_CalItem_Owner FOREIGN KEY (owner) REFERENCES users (user_id),
 CONSTRAINT CK_CalItem_Duration CHECK (duration >= 0)
 );
CREATE Index CI_RecType on calitem (rec_type);
CREATE TABLE calitem_calendar (
 cal_item_id int NOT NULL,
 calendar_id int NOT NULL,
 approved_state int DEFAULT 0 NOT NULL,
 approver int ,
 approve_date timestamp ,
 CONSTRAINT FK_CICal_CI FOREIGN KEY (cal_item_id) REFERENCES calitem (cal_item_id) ON DELETE CASCADE,
 CONSTRAINT FK_CICal_Cal FOREIGN KEY (calendar_id) REFERENCES calendar (calendar_id) ON DELETE CASCADE,
 CONSTRAINT PK_CICal PRIMARY KEY (cal_item_id, calendar_id)
);
create table mediaacct (
 account_id int NOT NULL PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY,
 account_type int NOT NULL,
 account_name VARCHAR(80) NOT NULL,
 authenticated boolean DEFAULT false NOT NULL,
 auth_data CLOB ,
 last_update timestamp DEFAULT CURRENT_TIMESTAMP NOT NULL,
 CONSTRAINT UQ_MA_AN UNIQUE (account_name)
 );
create table mediapost (
 mediapost_id int NOT NULL PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY,
 mediaaccount_id int NOT NULL,
 cal_item_id int NOT NULL,
 post_time timestamp NOT NULL,
 actually_posted timestamp ,
 process_flags smallint DEFAULT 0 NOT NULL,
 post_type int DEFAULT 0 NOT NULL,
 post_content CLOB ,
 post_metadata CLOB ,
 CONSTRAINT UQ_MP UNIQUE (mediaaccount_id,cal_item_id,post_time),
 CONSTRAINT FK_MAP_MA FOREIGN KEY (mediaaccount_id) REFERENCES mediaacct (account_id),
 CONSTRAINT FK_MAP_CI FOREIGN KEY (cal_item_id) REFERENCES calitem (cal_item_id)
 );
CREATE INDEX IX_MP_PROC ON mediapost (process_flags);
CREATE TABLE xmldocs (
 xmldoc_id int NOT NULL PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY,
 doc_type int NOT NULL,
 name VARCHAR(30),
 last_update timestamp NOT NULL,
 xml_doc CLOB NOT NULL
 );
CREATE TABLE caltabs (
 xmldoc_id int NOT NULL,
 calendar_id int NOT NULL,
 tab_order int NOT NULL,
 CONSTRAINT PK_CALTABS PRIMARY KEY (calendar_id,xmldoc_id),
 CONSTRAINT FK_CT_CAL FOREIGN KEY (calendar_id) REFERENCES calendar (calendar_id) ON DELETE CASCADE,
 CONSTRAINT FK_CT_XMLDOC FOREIGN KEY (xmldoc_id) REFERENCES xmldocs (xmldoc_id)
 );
CREATE TABLE calcalendar (
 calendar_id int NOT NULL,
 child_id int NOT NULL,
 display_opts int DEFAULT 0 NOT NULL,
 CONSTRAINT PK_CC PRIMARY KEY (calendar_id, child_id),
 CONSTRAINT FK_CC_C FOREIGN KEY (calendar_id) REFERENCES calendar (calendar_id) ON DELETE CASCADE
 ,CONSTRAINT FK_CC_CC FOREIGN KEY (child_id) REFERENCES calendar (calendar_id) ON DELETE CASCADE
 );
CREATE TABLE resourcetype (
 resource_type_id int NOT NULL PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY,
 parent_type int ,
 type_description VARCHAR(64) NOT NULL,
 attributes CLOB ,
 is_location int DEFAULT 0 NOT NULL,
 max_resources int DEFAULT 0 NOT NULL,
 CONSTRAINT FK_RT_RT FOREIGN KEY (parent_type) REFERENCES resourcetype (resource_type_id),
 CONSTRAINT UQ_ResourceType_Desc UNIQUE (Type_Description)
 );
CREATE TABLE resources (
 resource_id int NOT NULL PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY,
 active boolean DEFAULT true NOT NULL,
 resource_type_id int NOT NULL,
 location_id int ,
 resource_shared int DEFAULT 0 NOT NULL,
 resource_name VARCHAR(64) NOT NULL,
 descrmode int DEFAULT 0 NOT NULL,
 description CLOB ,
 attributes CLOB ,
 CONSTRAINT FK_RES_LOC FOREIGN KEY (location_id) REFERENCES location (location_id),
 CONSTRAINT FK_Resource_Type FOREIGN KEY (resource_type_id) references resourcetype (resource_type_id),
 CONSTRAINT UQ_Resources_Name UNIQUE (resource_name)
 );
CREATE TABLE resourcegroup (
 resourcegroup_id int NOT NULL PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY,
 group_name VARCHAR(64) NOT NULL,
 CONSTRAINT UQ_RG_GN UNIQUE (group_name)
 );
CREATE TABLE resourcegroupresource (
 resourcegroup_id int NOT NULL,
 resource_id int NOT NULL,
 CONSTRAINT PK_RGR PRIMARY KEY (resourcegroup_id,resource_id),
 CONSTRAINT FK_RGR_RES FOREIGN KEY (resource_id) references resources (resource_id) ON DELETE CASCADE,
 CONSTRAINT FK_RGR_RG FOREIGN KEY (resourcegroup_id) references resourcegroup (resourcegroup_id) ON DELETE CASCADE
 );
CREATE TABLE calitemresource (
 cal_item_id int NOT NULL,
 resource_id int NOT NULL,
 approved int DEFAULT 0 NOT NULL,
 CONSTRAINT PK_CalItemResource PRIMARY KEY (Cal_Item_id, Resource_id),
 CONSTRAINT FK_CIR_CalItem_id FOREIGN KEY (Cal_Item_id) References calitem (cal_item_id) ON DELETE CASCADE,
 CONSTRAINT FK_CIR_Resource_id FOREIGN KEY (Resource_id) References resources (resource_id)
 );
CREATE TABLE attachments (
 attachment_id int NOT NULL PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY,
 file_size int DEFAULT 0 NOT NULL,
 file_name VARCHAR(200) NOT NULL,
 def_description CLOB ,
 mime_type VARCHAR(64) ,
 md5sum VARCHAR(32) ,
 date_uploaded timestamp DEFAULT CURRENT_TIMESTAMP NOT NULL,
 CONSTRAINT CK_ATT_SIZE CHECK (file_size>=0)
 );
CREATE INDEX IX_ATT_FNAME on attachments (file_name);
CREATE INDEX IX_ATT_MIMEFNAME on attachments (mime_type,file_name);
CREATE TABLE calitemattachments (
 cal_item_id int NOT NULL,
 attachment_id int NOT NULL,
 file_name VARCHAR(200) NOT NULL,
 description CLOB ,
 CONSTRAINT PK_CIA PRIMARY KEY (cal_item_id,attachment_id),
 CONSTRAINT FK_CIA_CI FOREIGN KEY (cal_item_id) references calitem (cal_item_id) ON DELETE CASCADE,
 CONSTRAINT FK_CIA_AT FOREIGN KEY (attachment_id) references attachments (attachment_id)
 );
CREATE TABLE notificationuser (
 n_user_id int NOT NULL PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY,
 user_id int ,
 password VARCHAR(64) ,
 cookie VARCHAR(64) ,
 last_accessdate timestamp DEFAULT CURRENT_TIMESTAMP NOT NULL,
 confirmed smallint DEFAULT 0 NOT NULL,
 email_address VARCHAR(128) ,
 sms_number VARCHAR(64) ,
 reconfirmby int ,
 locales VARCHAR(64) ,
 timezone VARCHAR(64) ,
 failurecount int DEFAULT 0 NOT NULL,
 lastfailure timestamp ,
 lastsuccess timestamp ,
 CONSTRAINT FK_NU_UI FOREIGN KEY (User_ID) references users (user_id) ON DELETE CASCADE
 );
CREATE TABLE notificationqueue (
 queueentry_id int NOT NULL PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY,
 cal_item_id int NOT NULL,
 operator_id int NOT NULL,
 entry_time timestamp DEFAULT CURRENT_TIMESTAMP NOT NULL,
 create_time timestamp DEFAULT CURRENT_TIMESTAMP NOT NULL,
 entry_type int DEFAULT 0 NOT NULL,
 send_now int DEFAULT 0 NOT NULL,
 entry_date int NOT NULL,
 description VARCHAR(128) NOT NULL,
 in_process int DEFAULT 0 NOT NULL,
 list_built int DEFAULT 0 NOT NULL,
 op_notes CLOB ,
 CONSTRAINT FK_NQ_OP FOREIGN KEY (operator_id) REFERENCES users (user_id)
 );
CREATE INDEX IX_NQ_CI ON notificationqueue (cal_item_id);
CREATE TABLE notuserqueue (
 queueentry_id int NOT NULL,
 n_user_id int NOT NULL,
 effective_date int NOT NULL,
 processed int DEFAULT 0 NOT NULL,
 inprocess int DEFAULT 0 NOT NULL,
 processresult int DEFAULT 0 NOT NULL,
 ref_type int DEFAULT 0 NOT NULL,
 ref_id int ,
 metadata CLOB ,
 CONSTRAINT FK_NUQ_NQ FOREIGN KEY (queueentry_id) REFERENCES notificationqueue (queueentry_id) ON DELETE CASCADE,
 CONSTRAINT FK_NUQ_NU FOREIGN KEY (n_user_id) REFERENCES notificationuser (n_user_id) ON DELETE CASCADE,
 CONSTRAINT PK_NUQ PRIMARY KEY (QueueEntry_ID,N_User_ID)
 );
CREATE TABLE notuseritem (
 n_user_id int NOT NULL,
 cal_item_id int NOT NULL,
 notificationflags int DEFAULT 255 NOT NULL,
 days_offset int DEFAULT 0 NOT NULL,
 date_added timestamp DEFAULT CURRENT_TIMESTAMP NOT NULL,
 next_alarm timestamp ,
 day_list CLOB ,
 CONSTRAINT FK_NUI_NU FOREIGN KEY (n_user_id) REFERENCES notificationuser (n_user_id) ON DELETE CASCADE,
 CONSTRAINT PK_NUI PRIMARY KEY (n_user_id,cal_item_id),
 CONSTRAINT CK_NUI_DO CHECK (days_offset>=0 and days_offset<=30)
 );
CREATE TABLE notificationfilter (
 filter_id int NOT NULL PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY,
 n_user_id int NOT NULL,
 filtername VARCHAR(128) NOT NULL,
 searchfields int DEFAULT 2 NOT NULL,
 requirewords int DEFAULT 2 NOT NULL,
 filtersettype int DEFAULT 1 NOT NULL,
 includemerge int DEFAULT 0 NOT NULL,
 obj_type_id int NOT NULL,
 wordlist CLOB ,
 CONSTRAINT FK_NF_NU FOREIGN KEY (n_user_id) references notificationuser (n_user_id) ON DELETE CASCADE,
 CONSTRAINT FK_NF_OT FOREIGN KEY (obj_type_id) REFERENCES objecttypes (object_type_id)
 );
CREATE INDEX IX_NF_UI ON notificationfilter (n_user_id,filtersettype);
CREATE TABLE notificationfilterset (
 filter_id int NOT NULL,
 obj_type_id int NOT NULL,
 obj_id int NOT NULL,
 CONSTRAINT PK_NFS PRIMARY KEY (filter_id,obj_type_id,obj_id),
 CONSTRAINT FK_NFS_NF FOREIGN KEY (filter_id) REFERENCES notificationfilter (filter_id) ON DELETE CASCADE,
 CONSTRAINT FK_NFS_OT FOREIGN KEY (obj_type_id) REFERENCES objecttypes (object_type_id) ON DELETE CASCADE
 );
CREATE TABLE notification_others (
 n_user_id int NOT NULL,
 type_id int NOT NULL,
 entry_id int NOT NULL,
 entity_type int NOT NULL,
 entity_id int NOT NULL,
 CONSTRAINT FK_NO_NU FOREIGN KEY (n_user_id) REFERENCES notificationuser (n_user_id) ON DELETE CASCADE,
 CONSTRAINT FK_NO_TI_OT FOREIGN KEY (type_id) REFERENCES objecttypes (object_type_id),
 CONSTRAINT FK_NO_ET_OT FOREIGN KEY (entity_type) REFERENCES objecttypes (object_type_id),
 CONSTRAINT PK_NO PRIMARY KEY (n_user_id,type_id,entry_id,entity_type,entity_id)
 );
CREATE TABLE configuration (
 config_type int NOT NULL,
 last_updt timestamp NOT NULL,
 config CLOB NOT NULL,
 CONSTRAINT PK_CFG PRIMARY KEY (config_type)
 );
CREATE TABLE logentry (
 entry_id int NOT NULL PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY,
 object_type int NOT NULL,
 object_id int NOT NULL,
 twhen timestamp NOT NULL,
 user_name varchar(80),
 message CLOB
 );
CREATE INDEX IX_LE_OTOI ON logentry (object_type,object_id);
CREATE TABLE undoinfo (
 entry_id int NOT NULL PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY,
 user_id int NOT NULL,
 object_type int DEFAULT -1 NOT NULL,
 object_id int DEFAULT -1 NOT NULL,
 import_type int DEFAULT 0 NOT NULL,
 twhen timestamp NOT NULL,
 import_source CLOB ,
 insert_ranges CLOB ,
 undo_data CLOB ,
 CONSTRAINT FK_UI_U FOREIGN KEY (user_id) references users (user_id) ON DELETE CASCADE
 );
CREATE VIEW vObjectUsers AS
SELECT
 secured_obj_id,
 secured_obj_type,
 token,
 GU.group_id,
 CASE WHEN entity_type=2 THEN entity_id ELSE user_id END AS user_id,
 OT.type_name,
 OT.sort_order,
 OT.table_name,
 OT.field_name
FROM
 objectsecurity
 LEFT OUTER JOIN groupusers GU
 ON objectsecurity.entity_type=1 AND GU.group_id=objectsecurity.entity_id
 INNER JOIN objecttypes OT
 ON objectsecurity.secured_obj_type=OT.object_type_id;
CREATE VIEW vCalCalendar AS
SELECT
 CC.calendar_id,
 CC.child_id,
 CC.display_opts,
 CCP.name AS parent_name,
 CCC.Name AS child_name
FROM
 calcalendar CC
 INNER JOIN calendar CCP ON CCP.calendar_id=CC.calendar_id
 INNER JOIN calendar CCC ON CCC.calendar_id=CC.child_id;
CREATE VIEW vResource AS
SELECT
 R.resource_id,
 R.resource_type_id,
 R.active,
 R.location_id,
 R.resource_name,
 R.resource_shared,
 R.description,
 R.attributes,
 RT.type_description,
 LOC.location_name,
 RT.is_location
FROM
 resources R
 INNER JOIN resourcetype RT ON RT.Resource_Type_id=R.Resource_Type_id
 LEFT OUTER JOIN location LOC on LOC.location_id=R.location_id;
CREATE VIEW vCalItemResource AS
SELECT
 CIR.cal_item_id,
 CIR.resource_id,
 CIR.approved,
 RT.type_description,
 RT.is_location,
 R.resource_type_id,
 R.location_id,
 R.resource_name
FROM
 calitemresource CIR
 INNER JOIN resources R ON R.Resource_id=CIR.Resource_id
 INNER JOIN resourcetype RT ON RT.Resource_type_id=R.Resource_Type_id;
CREATE VIEW vCalItem AS
SELECT
 CI.*,
 case when U.Full_Name is null then U.User_Name else U.Full_Name end as owner_name
From
 calitem CI
 LEFT OUTER JOIN users U
 ON CI.owner=U.user_id;
CREATE VIEW vGroupUsers AS
SELECT
 GU.group_id,
 GU.user_id,
 G.group_name,
 U.user_name,
 U.full_name
FROM
 groupusers GU
 INNER JOIN groups G ON GU.group_id=G.group_id
 INNER JOIN users U ON GU.user_id=U.user_id;
CREATE VIEW vObjectSecurity AS
SELECT
 OSEC.*,
 OT.type_name,
 OT.sort_order AS type_sort_order,
 CT.sort_order AS token_sort_order
FROM
 objectsecurity OSEC
 INNER JOIN objecttypes OT
 ON OSEC.secured_obj_type=OT.object_type_id
 INNER JOIN candidatetokens CT
 ON OSEC.secured_obj_type=CT.object_type_id AND OSEC.token=CT.token;
CREATE VIEW vCandidateTokens AS
SELECT
 CT.*,
 OT.sort_order AS type_sort_order,
 CT.sort_order AS token_sort_order,
 OT.type_name
FROM
 candidatetokens CT
 INNER JOIN objecttypes OT
 ON CT.object_type_id=OT.object_type_id;
CREATE VIEW vNotUserItem AS
select NUI.*,
 CI.description
from notuseritem NUI
 LEFT JOIN calitem CI
 ON NUI.cal_item_id=CI.cal_item_id;
CREATE VIEW vResourceGroupResource AS
Select
 RGR.*,
 R.resource_name
from resourcegroupresource RGR
 INNER JOIN resources R
 ON R.Resource_ID=RGR.Resource_ID;
CREATE VIEW vMimeTypes as
select
 mime_type,
 count(*) as ref_count
from attachments
group by mime_type;
create view vCalGuestMap AS
select CGM.*,
 U.user_name
FROM
 calguestmap CGM
 INNER JOIN users U
 ON CGM.guest_user=U.user_id;
CREATE VIEW vCalTabs as
select
 caltabs.xmldoc_id,
 caltabs.calendar_id,
 caltabs.tab_order,
 xmldocs.xml_doc
from caltabs
 INNER JOIN xmldocs
 ON xmldocs.xmldoc_id=caltabs.xmldoc_id;
insert into cd_dbversion (version_str) values ('4.0.0');
INSERT INTO calitemtypes (Description) VALUES ('Concert');
INSERT INTO calitemtypes (Description) VALUES ('Meeting');
INSERT INTO calitemtypes (Description) VALUES ('Lecture');
INSERT INTO calitemtypes (Description) VALUES ('Class');
INSERT INTO calitemtypes (Description) VALUES ('Holiday');
INSERT INTO resourcetype (type_Description) VALUES ('A/V Equipment');
INSERT INTO resourcetype (type_Description,is_location) VALUES ('Facility',1);
INSERT INTO resources (Resource_Type_ID,Resource_Name,descrmode) values (2,'Conference Room',0);
INSERT INTO resourcetype (type_Description) VALUES ('Vehicle');
INSERT INTO calendar (Name) values ('Holidays');
INSERT INTO calendar (Name) values ('Events Calendar');
INSERT INTO calcalendar (calendar_id,child_id) values (2,1);
INSERT INTO objecttypes (object_type_id, type_name, sort_order, table_name, field_name)
 VALUES (32768, 'Calendar', 1, 'Calendar', 'calendar_id');
INSERT INTO objecttypes (object_type_id, type_name, sort_order, table_name, field_name)
 VALUES (32770, 'ResourceType', 3, 'ResourceType', 'resource_type_id');
INSERT INTO objecttypes (object_type_id, type_name, sort_order, table_name, field_name)
 VALUES (32771, 'Resources', 4, 'Resources', 'resource_id');
INSERT INTO objecttypes (object_type_id,type_name,sort_order,table_name,field_name)
 VALUES (32776,'ResourceGroup',5,'ResourceGroup','resourcegroup_id');
INSERT INTO objecttypes (object_type_id, type_name, sort_order, table_name, field_name)
 VALUES (32772, 'CalItemTypes', 6, 'CalItemTypes', 'item_type_id');
INSERT INTO objecttypes (object_type_id, type_name, sort_order, table_name, field_name)
 VALUES (32773, 'Users', 7, 'Users', 'user_id');
INSERT INTO objecttypes (object_type_id, type_name, sort_order, table_name, field_name)
 VALUES (32774, 'Groups', 8, 'Groups', 'group_id');
INSERT INTO objecttypes (object_type_id, type_name, sort_order, table_name, field_name)
 VALUES (32775, 'System', 9, null, null);
INSERT INTO objecttypes (object_type_id, type_name, sort_order, table_name, field_name)
 VALUES (32777,'Reminder',10,null,null);
INSERT INTO objecttypes (object_type_id, type_name, sort_order, table_name, field_name)
 VALUES (32778,'NotificationFilter',11,'NotificationFilter','filter_id');
INSERT INTO objecttypes (object_type_id, type_name, sort_order, table_name, field_name)
 VALUES (32779,'NotificationUser',12,'NotificationUser','n_user_id');
INSERT INTO objecttypes (object_type_id, type_name, sort_order, table_name, field_name)
 VALUES (32780,'CalItem',13,'CalItem','cal_item_id');
insert into objecttypes (object_type_id,type_name,sort_order,table_name,field_name) values
 (32782,'EMailMsg',14,null,null);
insert into objecttypes (object_type_id,type_name,sort_order,table_name,field_name) values
 (32784,'Location',2,'location','location_id');
insert into objecttypes (object_type_id,type_name,sort_order,table_name,field_name) values
 (32781,'SMSMessage',15,null,null);
insert into objecttypes (object_type_id,type_name,sort_order,table_name,field_name) values
 (32783,'MediaAcct',16,'mediaacct','account_id');
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32768, 0, 'tok_Add', 1,0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default,creator_default)
 VALUES (32768, 1, 'tok_Approve', 2,0,0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32768, 1, 'tok_AssignItemStyle', 3,0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32768, 1, 'tok_DefaultApproved', 4, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32768, 1, 'tok_Delete', 5, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32768, 1, 'tok_Edit', 6, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32768, 1, 'tok_EditOthersItems', 7, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default,creator_default)
 VALUES (32768, 1, 'tok_MailOnAdd', 8, 0, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default,creator_default)
 VALUES (32768, 1, 'tok_MailOnEdit', 9, 0, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32768, 1, 'tok_ManageSecurity', 10, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default,creator_default)
 VALUES (32768, 1, 'tok_RevokeOnEdit', 11, 0,0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32768, 1, 'tok_Use', 12, 1);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32768, 1, 'tok_View', 12, 1);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default,creator_default)
 VALUES (32768, 1, 'tok_Include', 13, 0, 1);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default,creator_default)
 VALUES (32768, 1, 'tok_OwnApproved', 14, 0, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32770, 0, 'tok_Add', 1, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32770, 1, 'tok_Delete', 2, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32770, 1, 'tok_Edit', 3, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32770, 1, 'tok_ManageSecurity', 4, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32770, 1, 'tok_View', 5, 1);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32770, 1, 'tok_Use', 6, 1);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32771, 0, 'tok_Add', 1, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32771, 1, 'tok_View', 1, 1);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32771, 1, 'tok_Use', 2, 1);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32771, 1, 'tok_DefaultApproved', 3, 1);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default,creator_default)
 VALUES (32771, 1, 'tok_Approve', 4, 0, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32771, 1, 'tok_Edit', 5, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32771, 1, 'tok_ManageSecurity', 5, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32771, 1, 'tok_Delete', 6, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32772, 0, 'tok_Add', 1, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32772, 1, 'tok_View', 2, 1);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32772, 1, 'tok_Use', 3, 1);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32772, 1, 'tok_Edit', 4, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32772, 1, 'tok_ManageSecurity', 5, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32772, 1, 'tok_Delete', 6, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32773, 0, 'tok_Add', 1, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32773, 0, 'tok_Delete', 2, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32773, 0, 'tok_Edit', 3, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,creator_default)
 VALUES (32773, 1, 'tok_ManageSecurity', 4, 1);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32774, 0, 'tok_Add', 1, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32774, 0, 'tok_Delete', 2, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32774, 0, 'tok_Edit', 3, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,creator_default)
 VALUES (32774, 1, 'tok_ManageSecurity', 4, 1);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32775, 0, 'tok_BulkException', 1, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32775, 0, 'tok_EditConfig', 2, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32775, 0, 'tok_HolidayAdd', 3, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32775, 0, 'tok_ManageItemStyles', 4, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32775, 0, 'tok_PurgeDB', 5, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32775, 0, 'tok_EditPrefs', 6, 1);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32775, 0, 'tok_CreateReminders', 7, 1);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32775, 0, 'tok_PermissionFixup', 8, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32775, 0, 'tok_ManageReminders', 9, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32775, 0, 'tok_CustomFieldDef', 10, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32777, 0, 'tok_AddOthersRem',1,0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32778, 0, 'tok_AddOthersFilt',1,0);
INSERT INTO candidatetokens (Object_Type_ID, instance, token, sort_order,is_default,creator_default)
 VALUES (32776,0,'tok_Add',1,0,0);
INSERT INTO candidatetokens (Object_Type_ID, instance, token, sort_order,is_default,creator_default)
 VALUES (32776,1,'tok_Edit',1,0,1);
INSERT INTO candidatetokens (Object_Type_ID, instance, token, sort_order,is_default,creator_default)
 VALUES (32776,1,'tok_Delete',2,0,1);
INSERT INTO candidatetokens (Object_Type_ID, instance, token, sort_order,is_default,creator_default)
 VALUES (32776,1,'tok_Use',3,1,1);
INSERT INTO candidatetokens (Object_Type_ID, instance, token, sort_order,is_default,creator_default)
 VALUES (32776,1,'tok_ManageSecurity',4,0,1);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32784, 0, 'tok_Add', 1, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32784, 1, 'tok_Delete', 2, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32784, 1, 'tok_Edit', 3, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32784, 1, 'tok_ManageSecurity', 4, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32784, 1, 'tok_View', 5, 1);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order,is_default)
 VALUES (32784, 1, 'tok_Use', 6, 1);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order, is_default)
 VALUES(32783, 0, 'tok_Add', 1, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order, is_default)
 VALUES(32783, 1, 'tok_Edit', 1, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order, is_default)
 VALUES(32783, 1, 'tok_Use', 2, 1);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order, is_default)
 VALUES(32783, 1, 'tok_Delete', 3, 0);
INSERT INTO candidatetokens (object_type_id, instance, token, sort_order, is_default)
 VALUES(32783, 1, 'tok_ManageSecurity', 4, 0);
INSERT INTO users (User_Name, Password, Full_Name, enabled) VALUES ('Admin', 'e3afed0047b08059d0fada10f400c1e5', 'Administrative User',1);
insert into users (User_Name, Password, Full_Name, Enabled, Default_Calendar, PlannerStart, PlannerEnd) values ('GUEST', 'DISABLED',' Guest User for Public Calendars', 0, 2, '9:00 AM', '5:00 PM');
INSERT INTO users ( User_Name, Password, Full_Name, enabled, Default_Calendar, PlannerStart, PlannerEnd) VALUES ('customer', 'e3afed0047b08059d0fada10f400c1e5', 'Customer Administrative Account', 1, 2, '9:00 AM', '5:00 PM');
insert into groups (Group_Name,Descr) values ('Users','Normal Users');
insert into groups (Group_Name,Descr) values ('Admins','Users with Administrative Powers');
insert into groupusers (Group_ID,User_ID)
 select groups.Group_ID,users.User_ID
 from users, groups
 Where groups.Group_Name='Users' and users.user_name<>'GUEST';
insert into groupusers (Group_ID,User_ID)
 select groups.Group_ID,users.User_ID
 from users,groups
 Where groups.Group_Name='Admins' AND users.user_name in ('Admin','customer');
Insert into objectsecurity (entity_id, entity_type, secured_obj_type, secured_obj_id, token)
 select groups.group_id, 1, candidatetokens.object_type_id, -1, candidatetokens.token
 from groups, candidatetokens
 where groups.Group_Name='Admins' AND
 candidatetokens.Instance=0;
Insert into objectsecurity (entity_id, entity_type, secured_obj_type, secured_obj_id, token)
 select groups.Group_ID, 1, candidatetokens.object_type_id, calitemtypes.item_type_id, candidatetokens.token
 from groups,calitemtypes, candidatetokens
 where groups.Group_Name='Admins' AND
 candidatetokens.Instance=1 AND
 candidatetokens.Object_Type_ID=32772 AND
 candidatetokens.creator_default=1;
Insert into objectsecurity (entity_id, entity_type, secured_obj_type, secured_obj_id, token)
 select groups.Group_ID, 1, candidatetokens.object_type_id, resourcetype.Resource_Type_ID, candidatetokens.token
 from groups,resourcetype, candidatetokens
 where groups.Group_Name='Admins' AND
 candidatetokens.Instance=1 AND
 candidatetokens.Object_Type_ID=32770 AND
 candidatetokens.creator_default=1;
Insert into objectsecurity (entity_id, entity_type, secured_obj_type, secured_obj_id, token)
 select groups.Group_ID, 1, 32771, resources.Resource_ID, candidatetokens.Token
 from groups,resources,candidatetokens
 where groups.Group_Name='Admins' AND
 candidatetokens.instance=1 AND
 candidatetokens.object_type_id=32771 AND
 candidatetokens.creator_default=1;
Insert into objectsecurity (entity_id, entity_type, secured_obj_type, secured_obj_id, token)
 select groups.Group_ID, 1, 32768, calendar.Calendar_ID, candidatetokens.Token
 from groups,calendar,candidatetokens
 where groups.Group_Name='Admins' AND
 candidatetokens.instance=1 AND
 candidatetokens.object_type_id=32768 AND
 candidatetokens.creator_default=1;
Insert into objectsecurity (entity_id, entity_type, secured_obj_type, secured_obj_id, token)
 select groups.Group_ID, 1, candidatetokens.object_type_id, calitemtypes.item_type_id, candidatetokens.token
 from groups,calitemtypes, candidatetokens
 where groups.Group_Name='Users' AND
 candidatetokens.Instance=1 AND
 candidatetokens.Object_Type_ID=32772 AND
 candidatetokens.is_default=1;
Insert into objectsecurity (entity_id, entity_type, secured_obj_type, secured_obj_id, token)
 select groups.Group_ID, 1, candidatetokens.object_type_id, resourcetype.Resource_Type_ID, candidatetokens.token
 from groups,resourcetype, candidatetokens
 where groups.Group_Name='Users' AND
 candidatetokens.Instance=1 AND
 candidatetokens.Object_Type_ID=32770 AND
 candidatetokens.is_default=1;
Insert into objectsecurity (entity_id, entity_type, secured_obj_type, secured_obj_id, token)
 select groups.Group_ID, 1, 32771, resources.Resource_ID, candidatetokens.Token
 from groups,resources,candidatetokens
 where groups.Group_Name='Users' AND
 candidatetokens.instance=1 AND
 candidatetokens.object_type_id=32771 AND
 candidatetokens.is_default=1;
Insert into objectsecurity (entity_id, entity_type, secured_obj_type, secured_obj_id, token)
 select groups.Group_ID, 1, 32768, calendar.Calendar_ID, candidatetokens.Token
 from groups,calendar,candidatetokens
 where groups.Group_Name='Users' AND
 candidatetokens.instance=1 AND
 candidatetokens.object_type_id=32768 AND
 candidatetokens.is_default=1 AND
 calendar.calendar_id=2;
Insert into objectsecurity (entity_id, entity_type, secured_obj_type, secured_obj_id, token)
 select groups.group_id, 1, 32775, -1, candidatetokens.Token
 from groups,candidatetokens
 where groups.Group_Name='Users' AND
 candidatetokens.instance=0 AND
 candidatetokens.object_type_id=32775 AND
 candidatetokens.is_default=1;
Insert into objectsecurity (entity_id, entity_type, secured_obj_type, secured_obj_id, token)
 select 2, 2, secured_obj_type, secured_obj_id,token
 from objectsecurity, groups
 where entity_id=groups.group_id AND
 entity_type=1 AND
 groups.group_name='Users';
delete from objectsecurity
 where entity_id=2 AND
 entity_type=2 AND
 secured_obj_type=32768 AND
 token = 'tok_Use';
CREATE FUNCTION passOneElimination( pRecType int, pStartDate int, pEndDate int, pRangeStart int,pRangeEnd int) RETURNS INT
 PARAMETER STYLE JAVA NO SQL LANGUAGE JAVA
 EXTERNAL NAME 'com.MHSoftware.db.support.DerbyDB.passOneElimination';
