SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO


CREATE  function dbo.passOneElimination(
	@iRecType int,
	@iStartDate int,
	@iEndDate int,
	@iRangeStart int,
	@iRangeEnd int)
returns int 
as
Begin
/*
Description

	This function does a pass one elimination as a SQL Function. The problem
	we're hitting with some clients is they have TONS of events in their system, 
	effectively all on the same calendar. By doing more filtering on the
	server, we reduce the number of returned records and dramatically improve
	performance.
	
Parameters

	@iRecType - The Connect Daily Record Type. For this, all we care about is
	4=Specific Dates which automatically we ignore, and the rest we process.

	@iStartDate - The start date of the Connect Daily Event. The format is a
		julian date, and may be null.
	@iEndDate - The end date of the Connect Daily Event.

	@iRangeStart - The Julian Date for the start of the range we care about.
	@iRangeEnd - The Julian date for the end date of the range we care about.

Returns

	1 if the record is within the range we're examining (or it's a specific dates event).
	0 if the record is outside the range we're looking at.

*/
declare @iResult int

select @iResult=0

		if (@iStartDate is null)
			select @iStartDate=-2147483648

		if (@iEndDate is null)
			select @iEndDate=2147483647

		if @iStartDate<@iRangeStart
			begin
				if @iRangeStart<=@iEndDate
					select @iResult=1
			end
		else
			begin
				if (@iStartDate=@iRangeStart OR @iStartDate<=@iRangeEnd)
					select @iResult=1
			end

return (@iResult)
end



GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS ON 
GO

