create or replace  function passOneElimination(
                pRecType number,
                pStartDate number,
                pEndDate number,
                pRangeStart number,
                pRangeEnd number)
return number
as
/*
Description
 
                This function does a pass one elimination as a SQL Function. The problem
                we're hitting is that some customers have TONS of events in their system, 
                effectively all on the same calendar. By doing more filtering on the
                server, we reduce the number of returned records improving performance
                and reducing GC overhead.
               
Parameters
 
                iRecType - The Connect Daily Record Type. For this, all we care about is
                4=Specific Dates which automatically we ignore, and the rest we process.
 
                iStartDate - The start date of the Connect Daily Event. The format is a
                               julian date, and may be null.
                iEndDate - The end date of the Connect Daily Event.
 
                iRangeStart - The Julian Date for the start of the range we care about.
                iRangeEnd - The Julian date for the end date of the range we care about.
 
Returns
 
                1 if the record is within the range we're examining (or it's a specific dates event).
                0 if the record is outside the range we're looking at.
 
*/
iResult number;
iStartDate number;
iEndDate number;
 
Begin
  iResult:=0;
  iStartDate:= nvl(pStartDate, -2147483648);
  iEndDate:= nvl(pEndDate, 2147483647);
  if (iStartDate<pRangeStart) then
     if (pRangeStart<=iEndDate) then
         iResult:=1;
     end if;
  else
     if (iStartDate=pRangeStart OR iStartDate<=pRangeEnd) then
        iResult:=1;
     end if;
  end if;   
return iResult;
end;
