/* This may throw an error if the language already exists. */
CREATE LANGUAGE plpgsql;

/*
Description

	This function does a pass one elimination as a SQL Function. The problem
	we're hitting with some clients is they have TONS of events in their system, 
	effectively all on the same calendar. By doing more filtering on the
	server, we reduce the number of returned records and dramatically improve
	performance.
	
Parameters

	pRecType - The Connect Daily Record Type. For this, all we care about is
	4=Specific Dates which automatically we ignore, and the rest we process.

	pStartDate - The start date of the Connect Daily Event. The format is a
		julian date, and may be null.
	pEndDate - The end date of the Connect Daily Event.

	pRangeStart - The Julian Date for the start of the range we care about.
	pRangeEnd - The Julian date for the end date of the range we care about.

Returns

	1 if the record is within the range we're examining (or it's a specific dates event).
	0 if the record is outside the range we're looking at.

*/


CREATE OR REPLACE FUNCTION passOneElimination (
        pRecType integer,
        pStartDate integer,
        pEndDate integer,
        pRangeStart integer,
        pRangeEnd integer
        ) RETURNS integer AS '
  DECLARE
     iResult integer := 0 ;
     iStartDate integer;
     iEndDate integer;
     INTEGER_MIN_VALUE CONSTANT integer := -2147483648;
     INTEGER_MAX_VALUE CONSTANT integer := 2147483647;
  BEGIN
     iStartDate:=coalesce(pStartDate,INTEGER_MIN_VALUE);
     iEndDate:=coalesce(pEndDate,INTEGER_MAX_VALUE);

     if iStartDate<pRangeStart then
             begin
                     if pRangeStart<=iEndDate then
                             iResult:=1;
                     end if;
             end;
     else
             begin
                     if iStartDate=pRangeStart OR iStartDate<=pRangeEnd then
                             iResult:=1;
                     end if;
             end;
     end if;
   RETURN iResult;
  END;' LANGUAGE 'plpgsql';
