/*
    Connect Daily Web Calendar Software

    Copyright 2014, MH Software, Inc. All Rights Reserved
    5023 W 120th Ave #311, Broomfield CO 80020
    +1 303 438 9585

	Last Edit By: "$Author: gsexton $"
	Last Checkin: "$Date: 2014/07/03 17:30:52 $"
	Revision #  : "$Revision: 1.1 $"
*/
/*
    This file contains javascript functions used by the Marquee page.
*/

/*
    Display Strategies - the displayStrategy value controls how the program creates the slides. Set
    the value of displayStrategy to one of the STRATEGY_ names shown below.

    STRATEGY_TODAY_ONLY
        Show only the current day's events. So here's a subtlety. If you set maxDaysForEvents to 
        multiple days, then the system will continue to run even if the network connection is 
        interrupted. IOW, if you set maxDaysForEvents to 3, and displayStrategy to 1, you can keep
        going for up to three days even if the net connection drops.

    STRATEGY_TODAY_FULL_SLIDE
        Show the current day plus enough events to finish filling the current slide.

    STRATEGY_ALL_RETURNED
        Show all events returned by the calendar. See Also: maxDaysForEvents

    STRATEGY_MAX_SLIDES
        Show all events returned up to a maximum of maxDisplaySlides

    STRATEGY_MAX_FUTURE_MINUTES -- TODO: Implement
        Show all events that start in the next maxFutureMinutes.
*/
var STRATEGY_TODAY_ONLY=1, STRATEGY_TODAY_FULL_SLIDE=2, STRATEGY_ALL_RETURNED=3, STRATEGY_MAX_SLIDES=4, STRATEGY_MAX_FUTURE_MINUTES=6;
/**
 *  This function performs the AJAX JSON request for the calendar data
 *
 *  @param url - the target URL to the calendar data.
 *  @param targetID - The ID of the HTML Element to receive the data.
 *  @param SingleDate - True if the data is for a single date.
 *  @param functionRef - callback function to actually process the data.
 *      it will receive the data, along with the targetID and SingleDate value.
 */
function ListEvents(url,targetID,SingleDate,functionRef){
    $.ajax({
           "url"      : url,
           "ifModified" : true,
           "cache"    : true,
           "dataType" : "jsonp",
           "error"    : function(jqXHR,textStatus,errorThrown){
               console.error("ListEvents - ajax error: "+errorThrown);
               },
           "success"  : function(data,textStatus,jqXHR){ 
               console.log("%d - %s",jqXHR.status,textStatus);
               if (jqXHR.status==304) {
                   console.info(" --------------- status 304 on update!");
                   eventSettings.lastDataUpdate=new Date().getTime();
               } else if (jqXHR.status==200){
                   data["TargetID"]=targetID;
                   data["SingleDate"]=SingleDate;
                   return functionRef(data,textStatus,jqXHR);
               } else {
                   console.info("Unhandled return code in ListEvents.success() method - status=%d",jqXHR.status);
               }
           }
        });
}

/**
 *  Get the location string for an event. Either it's location or it's location resource.
 * 
 *  @param item - The JSON event item to extract the location for.
 */
function getLocation(item) {
    if (item.hasOwnProperty('location') && item.location!==null) {
        return item.location.location_name;
    }
    var resources=item.resources;
    for (var r=0; r < resources.length; r++) {
        var thisResource=resources[r];
        if (thisResource.hasOwnProperty('location') && thisResource.location!==null) {
            return thisResource.location.location_name;
        }
        if (thisResource.is_location) {
            return thisResource.name;
        }
    } 
    return null;
}

var iLastJulian=-1;
/**
 *  Return the formatted time banner that should be displayed with this event.
 *
 *  @param item - The JSON event object to get the time banner for.
 *  @param pp - Process Parameter, holds state betwee calls.
 *  @param todayJulian - Today's Julian Day Number.
 *
 *  @return The formatted time banner, or an empty string if it's the same as
 *    the last call.
 */
function getTimeBanner(item,todayJulian) {
    
    var s="";
    console.log(" getTimeBanner('"+item.description+"',todayJulian="+todayJulian+" iLastJulian="+iLastJulian+" item.occurrenceStartJulian="+item.occurrenceStartJulian);
    // if it's today, AND the last one was the same julian date.
            if (todayJulian==item.occurrenceStartJulian && iLastJulian==item.occurrenceStartJulian) {   // For a single day, we only want to print the time if present.
                if (item.occurrenceStartTime===null) {
                } else {
                    s=item.occurrenceStartTime;
                }
            } else {   // For multi-day span lists, print the date and time.
                s=item.occurrenceStartDateTime;
            }
    iLastJulian=item.occurrenceStartJulian;
    return "<div class=\"StartTime\">"+s+"</div>";
}

/**
 *  Return today's julian date.
 */
function getTodayJulian() {
    
    var d=new Date(), 
        t=d.getTime()-d.getTimezoneOffset()*60000,
        days=(t/86400000)+2440588;
    return Math.floor(days);
}
/**
* Print out a detailed listing for a single event. This is what 
* actually formats and prints the details of an event. You can 
* add fields or change format here. 
*  
* @param item - the JSON event item to process
*
* @return the formatted HTML representing this single event.
*/
function processItemJSON(item) {
    var sLocation=getLocation(item);
    return "<div class=\"Title\" data-item-type-id=\""+item.item_type_id+"\">"+item.description+(sLocation===null ? '' : ' - '+sLocation)+"</div>";
}

/**
 *  this function creates metrics used by the slide creation routine. 
 *
 */
function createMetrics(es) {
    var Styles=["StartTime","Title"], Metrics={};
    console.log("body.height="+$("body").height());
    console.log("window.innerHeight="+window.innerHeight);
    console.log(es.targetDiv+".innerHeight()="+$(es.targetDiv).innerHeight());
    $.each(Styles,function(index,val){
            Metrics[val]=$("#"+val).outerHeight(true);
            console.log("Metrics['"+val+"']="+Metrics[val]);
        });
    es["metrics"]=Metrics;
}


/**
 * This function creates the slides. Essentially, it creates a slides array property
 * on eventSettings where each element is a slide of events to display.
 *
 * @param es  - Reference to EventSettings 
 * @param data - the JSON Data reference.
 */
function createSlides(es, data){
    console.log("createSlides - data.items.length="+data.items.length);
    es["eventData"]=data;

    var oldSlides=null;
    if (es.hasOwnProperty("slides")){
         oldSlides=es.slides;
     }
    es["slides"]=new Array();

    if (!es.hasOwnProperty("metrics")) {
        createMetrics(es);
    }

    if (!es.hasOwnProperty("slideState")) {
        es["slideState"]={
                "slideCount" : 0,
                "currentSlide" : 0,
                "intervalHandle" : 0,
                "divHeight" : $(es.targetDiv).innerHeight(),
                "incrementSlide" :  function(){     // Increment the slide number, returning the previous value.
                    var iLast=this.currentSlide++;
                    if (this.currentSlide>this.slideCount){
                       this.currentSlide=1;
                   }
                   return iLast;
                }
            };
    }
     if (!data.items[0].hasOwnProperty("html")) {
         /*
            Put the HTML representation of the item on the JSON object so we 
            don't have to recompute it all the time.
         */ 
         $.each(data.items,function(index,item) { item["html"]=processItemJSON(item); });
     }
     var todayJulian=getTodayJulian(), 
         iLastTime=-1,
         iPos=0,
         sSlide="";
         d=new Date(), 
         tCutoff=d.getTime()-es.cutoffMinutes*60000;

     iLastJulian=todayJulian;

     for (var iItem=0; iItem < data.items.length; iItem++) {
         var item=data.items[iItem];
         if (es.displayStrategy==STRATEGY_TODAY_ONLY && item.occurrenceStartJulian!==todayJulian){
             continue;
         } else if (item.occurrenceEnd<=tCutoff) {
             continue;
         } else if (item.occurrenceStartJulian>todayJulian && sSlide.length==0 && es.displayStrategy==STRATEGY_TODAY_FULL_SLIDE && es.slides.length>0){
             break;
         } else if (((typeof filterFunction) =='function') && filterFunction(item)) {
             continue;
         }

         var iThisItem=es.metrics.Title + (iLastTime==item.occurrenceStart ? 0 : es.metrics.StartTime);
         console.log("iThisItem="+iThisItem+" iPos="+iPos+" es.slideState.divHeight="+es.slideState.divHeight);
         if ((iThisItem+iPos)>es.slideState.divHeight) {
             console.log("Starting new Slide!");
             es.slides[es.slides.length]=sSlide;
             sSlide='';
             iLastTime=-1;
             iPos=0;
             if (
                 (es.displayStrategy==STRATEGY_MAX_SLIDES && es.slides.length>=es.maxDisplaySlides) ||
                 (es.displayStrategy==STRATEGY_TODAY_FULL_SLIDE && item.occurrenceStartJulian!=todayJulian)){
                 break;
             }
         }
         if (iLastTime!=item.occurrenceStart) {
             sSlide+=getTimeBanner(item,todayJulian);
         }
         sSlide+=item.html;
         iPos+=iThisItem;
         iLastTime=item.occurrenceStart;
     }
     if (sSlide.length > 0) {
         es.slides[es.slides.length]=sSlide;
     } else {
         if (es.slides.length==0){
             es.slides[0]="&nbsp;";
         }
     }
     es.slideState.slideCount=es.slides.length;
     var bChanged=(oldSlides==null) || (oldSlides.length!=es.slides.length);
     if (!bChanged) {
         for (var i=0; i < oldSlides.length; i++) {
             if (oldSlides[i]!=es.slides[i]) {
                 bChanged=true;
                 break;
             }
         }
     }
     es["slidesChanged"]=bChanged;
     if (bChanged) {
         es.slideState.currentSlide=0;
     }
     console.log("createSlides - slideCount="+es.slideState.slideCount+" bChanged="+bChanged);
     return es;
}



/**
 * This routine handles scrolling/slideshow operation of the returned data.
 */
function slideHandler(){
    var es=eventSettings,target=$(es.targetDiv),ss=es.slideState;

    /*
        Handle the slide transition.
    */
    
    var iPreviousSlide=ss.incrementSlide();

    console.log("previousSlide="+iPreviousSlide+" currentSlide="+ss.currentSlide+" slideCount="+ss.slideCount);

    if (iPreviousSlide!=ss.currentSlide || es.slidesChanged){
        console.log("Displaying slide "+ss.currentSlide+" of "+ss.slideCount+" changed="+es.slidesChanged);
        es.slidesChanged=false;
        $(target).fadeOut(es.slideOptions.fadeOutDurationMS,function(){
            $(target).html(es.slides[ss.currentSlide-1]);
            $(target).fadeIn(es.slideOptions.fadeInDurationMS);
            });
    }

    if (ss.intervalHandle==0){
        //  This is the first pass through, set up the refresher.
        console.info("setting interval call for slideHandler.");
        ss.intervalHandle=window.setInterval(slideHandler,ss.slideCount==1 ? 60000 : es.slideOptions.slideTimeSeconds*1000);
    } else {
        // This is not the first time the slideHandler has fired.
        if (ss.currentSlide==ss.slideCount){
            // We're displaying the last slide.
            if (haveNMinutesElapsedSince(es.lastDataUpdate,es.dataUpdateMinutes)){
                /*
                    The number of minutes between refreshes has elapsed, and we're on the last slide, kick the refresh.
                */
                console.info("Refresh Data Firing!");
                /* We need to tack the start julian date into the request so that we don't get 304's when we start a new day. */
                ListEvents(es.dataURL+"&start="+getTodayJulian()+"&dayspan="+(es.maxDaysForEvents-1),es.targetDiv,false,function(data,textStatus,jqXHR) {
                        window.clearInterval(es.slideState.intervalHandle);
                        es.slideState.intervalHandle=0;
                        console.info("Refresh data completed - Updated Data Received.");
                        es.lastDataUpdate=new Date().getTime();
                        createSlides(es,data);
                        slideHandler();
                    });
            } else {
                // Thumb through and see if there's a change because of aging using our existing data set.
                console.log("We're on the last slide, but enough time hasn't elapsed for a refresh. Re-examining current data.");
                createSlides(es,eventSettings.eventData);
            }

        }
    }
}

/**
 *  This is a timer function that puts the current time in the clock div.
 *
 *  @return void
 */
function clockFunc() {
    var d=new Date(),h=d.getHours(),m=d.getMinutes(),s=d.getSeconds(),
        sAMPM=(h>=12 ? "PM" : "AM");
    if (h==0) { 
        h=12; 
    } else if (h > 12) {
        h-=12;
    }

    var sTime=h+":"+(m<10 ? "0" : "")+m+" "+sAMPM;
    if (eventSettings.currentTime!=sTime){
        $(eventSettings.clockDiv).html(sTime);
        eventSettings.currentTime=sTime;
    }
}

/**
*
*  This pulls the current weather data from the weather channel using 
*  their JSON API. To signup, and get a Developer API key, go to:
*
*  http://www.wunderground.com/weather/api/
*
*  @return void
*/
function updateWeather(){
    $.ajax({
           "url"      : eventSettings.weatherAPIURL,
           "dataType" : "jsonp",
           "error"    : function(jqXHR,textStatus,errorThrown){
               console.error("updateWeather() - ajax error: "+errorThrown);
               $(eventSettings.weatherDiv).html("&nbsp;");
           },
           "success"  : function(data,textStatus,jqXHR){ 
               console.info("Received Weather Update");
               var cur=data.current_observation;
               var s='<img src="'+cur.icon_url+'">'+cur.weather+'<span style="margin-left: 2em">'+cur.temp_f+"&deg;</span>";
               $(eventSettings.weatherDiv).html(s);
           }
          });
}

/**
*  check if a specified number of minutes have elapsed.
*
*  @param sinceTime - UTC milli-Seconds since 1 Jan 1970
*  @param minutes - Number of minutes to check elapsed time for.
*
*  @return true if minutes have elapsed, false otherwise.
*/
function haveNMinutesElapsedSince(sinceTime,minutes){
    return new Date().getTime()>(sinceTime+minutes*60000);
}


/**
 * This function starts the slide show process, including the 
 * clock and weather display. 
 */
function startMarquee(eventSettings){
    ListEvents(eventSettings.dataURL+"&start="+getTodayJulian()+"&dayspan="+(eventSettings.maxDaysForEvents-1),
               eventSettings.targetDiv,
               false,
               function(data,textStatus,jqXHR){
                   if (eventSettings.displayStrategy==2 && eventSettings.maxDaysForEvents==1) {
                       // If we're padding out to a full slide, make sure we get at least some data to pad.
                       eventSettings.maxDaysForEvents=3;
                   }
                   console.log("listEvents - completion function fired!");
                   eventSettings.lastDataUpdate=new Date().getTime();
                   createSlides(eventSettings,data);
                   slideHandler();
            });
    if (eventSettings.enableClock){
        //  Update 5 times a seconds to keep the seconds part. if you drop the seconds part
        //  from the display, then this could be cut down to once a second or less.
        window.setInterval(clockFunc,2000);
    }
    if (eventSettings.weatherUpdateMinutes>0) {
        //  Since the interval is long, force it to run once before setting
        //  the interval.
        updateWeather();
        window.setInterval(updateWeather,eventSettings.weatherUpdateMinutes*60000);
    }
}
