/*
*   Last Edit By: "$Author: gsexton $"
*   Last Checkin: "$Date: 2009/04/18 19:19:34 $"
*/

/**
 *  This script detects changes in HTML forms. If the form is changed, navigating 
 * away will generate a warning out of the browser.
 * 
 * In your actual body onload, put in the call:
 * 
 * onload=&quot;return registerChangeHandler(frmTestChange,'You will lose your changes.');&quot;
 * 
 */
var fChanged=false;
var warningMessage="The form data has changed. If you navigate away, your changes will be lost.";

function setChangeWarningMessage(sWarningMessage){
    warningMessage=sWarningMessage;
    return true;
}


/** 
 * Register an event handler for an HTML Element. Based on code from The 
 * JavaScript Anthology by James Edwards &amp; Cameron Adams.
 * 
 * @param target - The HTML element to hook the event to.
 * @param eventType - The name of the Event. E.G. &quot;load&quot;
 * @param functionRef - The function reference to act as the event handler.
 * @param capture - True to capture the event.
 */
function attachEventListener(target, eventType, functionRef,capture){
    if (typeof target.addEventListener!="undefined") {
        target.addEventListener(eventType,functionRef,capture);
    } else if (typeof target.attachEvent !="undefined") {
        target.attachEvent("on"+eventType,functionRef);
    } else {
        eventType="on"+eventType;
        if (typeof target[eventType]=="function") {
            var oldListener=target[eventType];
            {
                oldListener();
                return functionRef();
            }
        } else {
            target[eventType]=functionRef;
        }
    }
}

function changeSet(param){
    fChanged=true;
    return true;
}

function unsetChanged(){
    fChanged=false;
    return true;
}

function isChanged(){
    return fChanged;
}

function manualWarn(){
    if (!isChanged()) {
	return true;
    }
    return confirm(warningMessage);
}

function unloadHandler(evt){
    if (fChanged) {
        if (typeof evt == 'undefined') {
            evt = window.event;
        }
        evt.returnValue=warningMessage;
    }
}

/** 
 * Register the change handler. The theory of operation is to attach change handlers to each 
 * form element. Anytime the change event fires, it sets the changed flag.
 * <BR><BR>
 * The one unfortunate side-effect here is that you have to unset the changed flag
 * in your form's onsubmit code. Essentially, call the unsetChanged() function.
 * 
 * 
 * @param formRef - Reference to the form we're hooking.
 * @param sWarning - The text warning message to display.
 */
function registerChangeHandler(formRef,sWarning){
    for (var i=0; i < formRef.elements.length; i++) {
        attachEventListener(formRef.elements[i],"change",changeSet,false);
    }
    attachEventListener(window,"beforeunload",unloadHandler,true);
    attachEventListener(formRef,"submit",unsetChanged,false);
    if (sWarning!=null) {
        setChangeWarningMessage(sWarning);
    }
}


