/* 
	This file contains javascript methods used for validation of SMS Text Numbers.

	Connect Daily Web Calendar Software

	Copyright 2014, MH Software, Inc. All Rights Reserved
	5023 W 120th Ave #311, Broomfield CO 80020
	+1 303 438 9585

	Last Edit By: "$Author: gsexton $"
	Last Checkin: "$Date: 2012/12/01 18:12:30 $"
	Revision #  : "$Revision: 1.4 $"
*/

/** 
 * Dialog 
 *  
 * IDDialogValidateSMS 
 *  
 * Form Elements Are 
 *  
 * IDValidateValue 	-	Text Box
 *  
 * IDValidateSend 	-	Buttons
 * IDValidateOK 
 * IDValidateCancel 
 *  
 */

/**
 * This function sends the SMS Validation code to the handset. 
 * It does it by making a JSON request back to the server.
 * 
 * @author gsexton (4/25/2014)
 */
function sendSMSValidation(){
	console.log("sendSMSValidation() invoked.");
	require(["dojo/request/xhr","dojo/dom","dojo/dom-attr"],function(xhr,dom,domAttr){
		var sValue=domAttr.get('IDsms_number','value');
		domAttr.set("IDValidateSend","disabled","");
		xhr("json/smsValidation.js",
				{
				"method" : "GET",
				"sync" : true,
				"preventCache": true,
				"query" : {
						"request_number": 9,
						"phone": sValue
						},
				"handleAs" : "json"
				}
				).then(function(data){
					// Return has: result, message, validationCode
					if(data.result===0) {
						domAttr.set('IDValidateValue','data-validation-code',data.validationCode);
						domAttr.remove('IDValidateOK',"disabled");
						domAttr.remove('IDValidateValue',"disabled");
						document.getElementById('IDValidateValue').focus();
					} else {
						if (data.result===114) {
							// This is a clickatell routing error which means
							// it's either an invalid #, or it's out of service area.
							domAttr.set("IDsms_number","value","");
						} else {
							domAttr.remove("IDValidateSend","disabled");
						}
						alert(data.message);
					}
				});
		});

}

/**
 * This function compares the validation code entered by the 
 * user to the one returned by sendSMSValidation. 
 */
function checkSMSValidation(){
	require(["dojo/dom-attr","dijit/registry"],function(domAttr,registry){
		var bResult=(domAttr.get("IDValidateValue","value")===domAttr.get("IDValidateValue","data-validation-code"));
		if (bResult) {
			domAttr.set("IDsms_number","data-original-value",domAttr.get("IDsms_number","value"));
			var dlg=registry.byId("IDDialogValidateSMS");
			dlg.hide();
		} else {
			alert(domAttr.get('IDValidateValue','data-validation-error'));
		}
		return bResult;
	});
}



/**
 * This function compares the value, and data-original-value 
 * attribute of the sms_number text box and if they're 
 * different, it returns true. 
 *  
 * @param elementID 
 */
function isSMSValidationRequired(elementID,popIfTrue){
	function clnPhone(val){
		if (val===null) {
			return '';
		}
		var s='';
		for (var i=0; i < val.length; i++) {
			var c=val.charAt(i);
			if (c>='0' && c<='9' ) {
				s+=c;
			} else if (c==='+') {
				s+=c;
			}
		}
		return s;
	}
	var bResult=false;

	require(["dojo/dom-attr"],function(domAttr){
		var sValue=domAttr.get(elementID,"value"),sCurrent=clnPhone(sValue), sOriginal=clnPhone(domAttr.get(elementID,"data-original-value"));
		if (sCurrent.length==10 && sCurrent.charAt(0)!='+') {
			sValue='1 '+sValue;
			domAttr.set(elementID,"value",sValue);
			sCurrent='1'+sCurrent;
			sOriginal="~";
		}
		bResult=(sCurrent.length>0) && !(sCurrent===sOriginal);
	});
	if (bResult && popIfTrue) {
		showSMSValidationDialog();
	}
	return bResult;
}

/**
 * Display the SMS Validation Dialog
 */
function showSMSValidationDialog() {
	require(["dijit/registry","dojo/dom-attr"],function(reg,domAttr){

		domAttr.set('IDValidateValue','value','');
		domAttr.set('IDValidateValue','disabled','');
		domAttr.remove('IDValidateSend',"disabled");
		domAttr.set('IDValidateOK',"disabled","");

		var dlg=reg.byId('IDDialogValidateSMS');
		dlg.show();
	});
}
